
file_id = "SPI_7SEGLED"

--[[

●機能概要

SPI チャンネルに接続した 7セグメントLEDマトリックスに文字列を表示する。

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com			Arduino が接続された COM ポート名またはタイトル名			"COM4"

data		LEDマトリックスに表示する文字列データ						"12345678"
			数値以外もできるだけ7セグメントLED に表示できるように
			変換する

clear		このパラメータを指定すると最初にLCD 表示内容をクリアする
			値は何でも構わない											"1"

width		表示器の桁数を指定して data パラメータに指定した表示文字数が
			witdth に満たない場合には左部分を空白で埋める				"8"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------


●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[SPI_WRITE] をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

7セグメントLED ビット対応図
---------------
MSB         LSB
---------------
P G F E D C B A
---------------

7セグメントLEDセグメント配置

  A
  --
F|__|B
E|G |C
  --   .P
  D

●変更履歴

2015/05/25	桁数を指定できるようにした
2014/05/21	ピリオドを表示できるようにした
2014/04/23	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

file_id = "SPI_LED_MATRIX"

-------------------------
-- パラメータチェック
-------------------------
if not g_params["com"] then
	log_msg("parameter error",file_id)
	error()
end

-----------------------------------------------------------
-- 1文字をセグメントデータパターンを表す1バイト16進数に変換
-----------------------------------------------------------
function char_to_seg(chr)
	local ret = 0x0
	if (chr == "0") or (chr == "O") then
		ret = 0x3f
	elseif (chr == "1") or (chr == "I") then
		ret = 0x06
	elseif (chr == "2") then
		ret = 0x5b
	elseif (chr == "3") then
		ret = 0x4f
	elseif (chr == "4") then
		ret = 0x66
	elseif (chr == "5") or (chr == "S") or (chr == "s") then
		ret = 0x6d
	elseif (chr == "6") then
		ret = 0x7d
	elseif (chr == "7") then
		ret = 0x27
	elseif (chr == "8") then
		ret = 0x7f
	elseif (chr == "9") then
		ret = 0x6f
	elseif (chr == "a") or (chr == "A") then
		ret = 0x77
	elseif (chr == "b") or (chr == "B") then
		ret = 0x7c
	elseif (chr == "C") then
		ret = 0x39
	elseif (chr == "c") then
		ret = 0x58
	elseif (chr == "d") or (chr == "D") then
		ret = 0x5e
	elseif (chr == "E") then
		ret = 0x79
	elseif (chr == "e") then
		ret = 0x7b
	elseif (chr == "f") or (chr == "F") then
		ret = 0x71
	elseif (chr == "g") or (chr == "G") then
		ret = 0x3d
	elseif (chr == "H") then
		ret = 0x76
	elseif (chr == "h") then
		ret = 0x74
	elseif (chr == "I") then
		ret = 0x06
	elseif (chr == "i") then
		ret = 0x04
	elseif (chr == "J") then
		ret = 0x1e
	elseif (chr == "j") then
		ret = 0x0e
	elseif (chr == "k") or (chr == "K") then
		ret = 0x78
	elseif (chr == "l") or (chr == "L") then
		ret = 0x38
	elseif (chr == "m") or (chr == "M") then
		ret = 0x55
	elseif (chr == "n") or (chr == "N") then
		ret = 0x54
	elseif (chr == "o") then
		ret = 0x5c
	elseif (chr == "p") or (chr == "P") then
		ret = 0x73
	elseif (chr == "q") or (chr == "Q") then
		ret = 0x67
	elseif (chr == "r") or (chr == "R") then
		ret = 0x50
	elseif (chr == "t") or (chr == "T") then
		ret = 0x46
	elseif (chr == "U") then
		ret = 0x3e
	elseif (chr == "u") then
		ret = 0x1c
	elseif (chr == "v") or (chr == "V") then
		ret = 0x18
	elseif (chr == "w") or (chr == "W") then
		ret = 0x1d
	elseif (chr == "x") or (chr == "X") then
		ret = 0x36
	elseif (chr == "y") or (chr == "Y") then
		ret = 0x6e
	elseif (chr == "z") or (chr == "Z") then
		ret = 0x49
	elseif (chr == "-") then
		ret = 0x40
	elseif (chr == "_") then
		ret = 0x08
	elseif (chr == ".") then
		ret = 0x80
	elseif (chr == "~") then
		ret = 0x01
	else
		ret = 0x0
	end
	return bit_and(bit_not(ret),0xff) -- LED 負論理の為パターンを反転
end

-------------------------------------------------------------
-- 文字列データを1文字ごとにセグメントパターンデータに変換して
-- SPI チャンネルに出力する
-------------------------------------------------------------
local tbl = {}
local seg_data = {}
local ch = ""
local ch_int = 0
local wp = 1
local data = ""

tbl["com"] = g_params["com"]

------------------------------------
-- 全てのセグメントを消灯
------------------------------------
if g_params["clear"] then
	tbl["data"] = "FFFFFFFFFFFFFFFF" -- 負論理でLEDが接続されている
	if not script_exec("ARDUINO/SPI_WRITE",tbl) then error() end
end

------------------------------------
-- 指定された文字をセグメントに表示
------------------------------------
if g_params["data"] and (g_params["data"] ~= "") then
	for i=1,#g_params["data"],1 do
		ch = string.sub(g_params["data"],i,i)
		ch_int = char_to_seg(ch)
		-------------------------------------------------
		-- ピリオドをひとつ前のセグメントに重ねて表示する
		-------------------------------------------------
		if ch == '.' then
			if wp > 1 then
				seg_data[wp-1] = bit_and(seg_data[wp-1],ch_int)
			else
				seg_data[wp] = ch_int
				wp = wp + 1
			end
		else
			seg_data[wp] = ch_int
			wp = wp + 1
		end
	end
	------------------------------------------------------------
	-- SPI に送信するバイト列をセグメント表示器用に前後入れ替え
	------------------------------------------------------------
	for i=1,#seg_data,1 do
		data = bit_tohex(seg_data[i],2) .. data
	end
	---------------------------------------------------------------------
	-- 表示器の桁数を指定している場合には左余白をブランクで埋める
	---------------------------------------------------------------------
	if g_params["width"] then
		local width = tonumber(g_params["width"])
		if width > #seg_data then
			for i=1,(width - #seg_data),1 do
				data = data .. "FF"
			end
		end
	end

	tbl["data"] = data
	if not script_exec("ARDUINO/SPI_WRITE",tbl) then error() end
end


