
file_id = "ARDUINO/I2C_READ"

--[[

●機能概要

Arduino I2C バスに接続したスレーブデバイスから指定バイト数分のデータ読み込みを行う

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				Arduino が接続された COM ポート名またはタイトル名		"COM4"

slave_addr		I2C スレーブデバイスアドレス(16進数2桁)					"50"

count			データ読み込みバイト数(32 以下の10進整数)				"32"


●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
I2C_DATA		スレーブで倍しから読み込んだデータ(16進数)列			
				最大32 バイト(64文字)まで								"0A0B0C

●備考

読み込みデータ数については、Arduino 側で動作させる FIRMATA スケッチプログラムの
仕様を参照してください。ここで指定した最大数以外にI2C スレーブデバイス毎の仕様に
よって制限がある場合がありますので注意してください。

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[I2C_READ_QUERY] [I2C_READ_REPLY] をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

●変更履歴

2017/06/28	abs_agent 用に排他制御コール部分を変更

2014/04/03	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not (g_params["com"] and g_params["slave_addr"] and g_params["count"]) then
	log_msg("parameter error",file_id)
	error()
end

----------------------------------------------------------------------
-- Firmata I2C_READ_QUERY リクエストフレーム作成
----------------------------------------------------------------------
local com = g_params["com"]
local sysex_start = "F0"
local sysex_end = "F7"
local firmata_request = "68"
local firmata_reply = "66"
local slave_addr_hexstr = g_params["slave_addr"]
local read_count_hexstr = list_to_hex(tonumber(g_params["count"]))
local send_data = sysex_start ..  firmata_request .. slave_addr_hexstr .. read_count_hexstr .. sysex_end

----------------------------------------------------------------------
-- 排他制御開始
-- 同じ Arduino デバイスに対して、I/O PORT QUERY コマンドが複数同時実行される場合に備えています。
-- 別のArduino デバイスを操作する場合や、同じ Arduino デバイスに対して異なる FIRMATA REPLY
-- コマンドバイト値を取得する場合には排他制御の必要はありません
----------------------------------------------------------------------
--local cstat,handle = critical_section_enter("Firmata" .. firmata_request,5000);
local cstat = critical_section_enter2("Firmata" .. firmata_request,5000);
if not cstat then
	log_msg("critical_section_enter() failed",file_id)
	error()
end

----------------------------------------------------------------------
-- リクエストフレーム送信とリプライ受信
----------------------------------------------------------------------
local stat,rdata = serial_write(com,send_data,firmata_reply)

----------------------------------------------------------------------
-- 排他制御終了
----------------------------------------------------------------------
-- cstat = critical_section_leave(handle)
cstat = critical_section_leave2("Firmata" .. firmata_request)
if not cstat then
	log_msg("critical_section_leave() failed",file_id)
	error()
end
if not stat then
	log_msg("serial_write() failed",file_id)
	error()
end


---------------------------------------------------------------
-- SYSEX コマンド I2C_READ_REPLY 受信パケット処理
---------------------------------------------------------------
local msg = ""	
local tmp = string.sub(rdata,5,-3);
if tmp ~= "" then
	local data = hex72_to_tbl(tmp,2)
	if data ~= nil then
		for key,val in ipairs(data) do
			msg = msg .. bit_tohex(val,2)
		end
	end
	if not script_result(g_taskid,"I2C_DATA",msg) then error() end
end



