
--[[

●機能概要

FASTDB で排他ロック中のキーが一定期間以上存在しつづけている場合に、
強制的にロック状態を解除する


●リクエストパラメータ

---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------

lock_timeout	最初にロック状態を確認してから、lock_timeout(秒) 以上経過しても
				依然ロック中のキーを発見した場合にアンロック対象とする。
				このパラメータを省略した場合には "60" が使用される
													

●リターンパラメータ

なし

●備考

このスクリプトは PERIODIC_TIMER イベントハンドラ等から定期的に別スレッドとして
実行してください。


●変更履歴

2020/11/4	  exclusive_check() による2重起動禁止コードを追加

2018/07/06    初版作成

abs_agent	copyright(c) All Blue System

]]

-- ２重起動防止用チェック
if not exclusive_check(g_script) then
	log_msg("*ERROR* exclusive_check() failed. script = " .. g_script,g_script)
	return
end

local lock_timestamp_prefix = '$FASTDB_LOCK_'
local lock_timeout = 60
if g_params["lock_timeout"] then
    lock_timeout = tonumber(g_params["lock_timeout"])
end

-- ロック中のキー名を使用したフラグ(グローバル共有変数)に保存するタイムスタンプ文字列を作成
local now = os.date "*t"
local timestamp = string.format("%4.4d/%2.2d/%2.2d %2.2d:%2.2d:%2.2d",now["year"],now["month"],now["day"],now["hour"],now["min"],now["sec"])

-- 全キー名のテーブルのインデックスに使用したテーブルを作成。テーブルの値にはロック中かどうかを示すフラグを格納する
local key_stat = {}
local key_list
key_list = stat_check(fastdb_key_list()) -- 全キー取得
for key,val in ipairs(key_list) do
	key_stat[val] = "free"
end
key_list = stat_check(fastdb_key_list(true)) -- ロック中のキーのみ取得
for key,val in ipairs(key_list) do
	key_stat[val] = "lock"
end

-- FASTDB で使用中のキーに対して下記の処理を実行
for key,val in pairs(key_stat) do
	if val == "free" then
		-- 今回コールするよりも前にロック済みにマークされて、タイムスタンプを保存していた場合には削除する
		if not set_shared_data(lock_timestamp_prefix .. key,"") then error() end
	else
		local old_timestamp = stat_check(get_shared_data(lock_timestamp_prefix .. key))
		if old_timestamp ~= "" then
			-- 今回コールするよりも前にロック済みだった場合にはタイムスタンプが lock_timeout 以上経過していないかをチェックする
			local o_year,o_month,o_day,o_hour,o_min,o_sec = stat_check(str_to_datetime(old_timestamp))
			local elapsed_sec = stat_check(seconds_between(o_year,o_month,o_day,o_hour,o_min,o_sec,
												now["year"],now["month"],now["day"],now["hour"],now["min"],now["sec"]))
			if elapsed_sec > lock_timeout then -- lock_timeout 以上経過したロック中のキーを強制的にアンロックする
				log_msg("*WARNING* force unlock: " .. key,g_script)
				stat_check(fastdb_unlock(key))
				stat_check(set_shared_data(lock_timestamp_prefix .. key,""))
			end
		else 
			-- 今回コールするよりも前にはロックされていなかったので新規にタイムスタンプを保存する
			stat_check(set_shared_data(lock_timestamp_prefix .. key,timestamp))
		end
	end
end
