
--[[

●機能概要

MGCP ファームウエアを搭載した M5Stack/M5Stick-C デバイス LCD にメッセージを表示する。
(MGCPデバイスが M5Stick-C/M5Stack 以外の場合にはなにも行わない)
ノードごとに保存された過去メッセージを再表示可能。
メッセージ表示と同時にデバイス内蔵の LED 点滅やブザーを鳴らすことができる。
これらの機能はスクリプト中の変数 (use_led_blink, use_buzzer) で有効・無効を設定できる。

●リクエストパラメータ

----------------------------------------------------------------------------------------------------------------
キー値			値		            									値の例
----------------------------------------------------------------------------------------------------------------
node		操作対象のMGCPデバイスノード名							"STICK001"
			パラメータ省略時は全ノードを対象にする

data		LCD に表示する文字列									"Hello World!!これは試験です"
			長い文字列は自動的に折り返して表示する。
			デバイスに表示する内容は "n/n <data> HH:MM(曜日)" になる。
			n/n 部分はノードごとのメッセージリスト保存数で HH:MM(曜日) はタイムスタンプ。
prev
next	
			最後に表示したメッセージから１つ前または１つ後のメッセージを表示する。
			データ値は何でも構わない。node パラメータを同時に指定すること。
			リストの最初または最後のメッセージ表示後は、リストの最後または先頭のメッセージを表示する。
			このパラメータ指定を行った１回目のコール時に、対象 node がLED点滅またはブザー出力状態の場合は
			その停止動作のみを行う。
																	"1"

clear		LCD 画面をクリアする。データ値にカラーコードを指定する。	"1"
			デバイス毎に保存されている過去メッセージも消去する。デバイスのLED点滅やブザー出力状態も停止する。
			node パラメータにデバイスノード名を指定する。node パラメータ省略時や "" 空文字列指定時は全ノードを対象にする。

			"0" は黒色で "1"は NAVY色 にアサインされています。その他のカラーコードは以下の通り
			[0] BLACK [1] NAVY [2] DARKGREEN [3] DARKCYAN [4] MAROON [5] PURPLE [6] OLIVE [7] LIGHTGREY
			[8] DARKGREY[9] BLUE [10] GREEN [11] CYAN [12] RED [13] MAGENTA [14] YELLOW [15] WHITE
			[16] ORANGE [17] GREENYELLOW [18] PINK [19..29] user preset color

attn		LED 点滅またはブザー出力を指定する。省略時はスクリプト中のデフォルト値を使用する。
			データ値は何でも構わない。data パラメータと同時に指定すること。
																	"1"

●変更履歴

2021/7/16	attn オプション追加

2021/6/30	初版作成

abs_agent	copyright(c) 2021 All Blue System

]]

local use_led_blink = true						-- M5Stick-C デバイスでメッセージ表示時に LEDを点滅させる
local use_buzzer = false						-- M5Stack デバイスでメッセージ表示時にブザーを鳴らす

if g_params["attn"] then
	use_led_blink = true
	use_buzzer = true
end

local ch_prefix = "M5_LCD_MSG_LIST_"			-- 各ノード毎に表示したメッセージ
local blink_flag_pre = "M5_LCD_BLINK_FLAG_"	-- LEDブリンク状態を示すフラグ
local buzzer_flag_pre = "M5_LCD_BUZZER_FLAG_"	-- ブザー出力状態を示すフラグ
local msg_ptr_pre = "M5_LCD_LIST_IDX_"			-- 過去メッセージ表示位置
local max_msg_list = 30						-- 過去メッセージ最大保存数
local split = 20
local font = "SHINONOME_FONTX"
local fg_color = 15
local bg_color = 1

local p_node = ""
local p_data = ""
if g_params["node"] then
	p_node = g_params["node"]
end
if g_params["data"] then
	p_data = g_params["data"]
end

-- このスクリプトが操作対象とする MGCP デバイスノード名リスト
local target_node_list = {} 

-- 指定したノードを操作対象リストに追加する
-- リストの各エントリにはノード名とデバイスタイプ文字列のペアを格納する
function add_node_type_list(node)
	local type_str = stat_check(get_shared_data('$TYPE_' .. node))
	table.insert(target_node_list,{node_name = node,node_type = type_str}) 
end

-- abs_agent が保持している最新の全ノードリストを操作対象にする
function build_all_node_list()
	local node_list = stat_check(get_shared_strlist('$MGCP_NodeList'))
	for i,v in ipairs(node_list) do add_node_type_list(v) end
end

-- ノード名パラメータを省略した場合は全ノードを操作対象にする
if p_node == "" then
	build_all_node_list()
else
	add_node_type_list(p_node)
end

-- LCDクリアとノード毎に保存されているメッセージリストを消去
function clear_msg_history(node)
	-- 過去メッセージ削除
	stat_check(set_shared_data(msg_ptr_pre .. node,""))
	stat_check(clear_shared_strlist(ch_prefix .. node))
	-- 画面クリア
	local cmd = {}
	cmd["command"] = "lcd_print"
	cmd["text"] = "---"
	cmd["fg"] = fg_color
	cmd["bg"] = bg_color
	cmd["clear"] = tonumber(g_params["clear"])
	mgcp_command(node,g_json.encode(cmd))
end

-- ノードがLED点滅状態の時にLEDを消灯して true を返す。
function blink_mode_clear(node,force)
	local flag = stat_check(get_shared_data(blink_flag_pre .. node))
	if force or (flag ~= "") then
		stat_check(set_shared_data(msg_ptr_pre .. node,""))
		stat_check(set_shared_data(blink_flag_pre .. node,""))
		script_fork_exec("MGCP/DEVICE/M5_STICK_LED","node,state",node .. "," .. "off")
		return true
	end
	return false
end

-- ノードのブザーが出力中の時は停止して true を返す。
function buzzer_mode_clear(node,force)
	local flag = stat_check(get_shared_data(buzzer_flag_pre .. node))
	if force or (flag ~= "") then
		stat_check(set_shared_data(msg_ptr_pre .. node,""))
		stat_check(set_shared_data(buzzer_flag_pre .. node,""))
		script_fork_exec("MGCP/DEVICE/M5_STACK_BUZZER","node,state",node .. "," .. "off")
		return true
	end
	return false
end

------------------------------------------------
-- LCD 画面クリア,ブザー停止、LED消灯・コマンド
------------------------------------------------
if g_params["clear"] then
	for i,v in ipairs(target_node_list) do
		if v.node_type == "M5Stack" then	-- M5Stack
			clear_msg_history(v.node_name)
			buzzer_mode_clear(v.node_name,true)
		end
		if v.node_type == "M5Stick" then	-- M5Stick-C
			clear_msg_history(v.node_name)
			blink_mode_clear(v.node_name,true)
		end
	end
end

-- 指定ノードのLCDに文字列表示とメッセージリスト登録。
function disp_msg_and_store_history(node,str,fnt_size)
	-- メッセージに追加するタイムスタンプ文字列作成
	local now = os.date "*t"
	local wday = stat_check(day_of_week(now["year"],now["month"],now["day"]))
	local weekstr
	if wday == 1 then 
		weekstr = "日"
	elseif wday == 2 then
		weekstr = "月"
	elseif wday == 3 then
		weekstr = "火"
	elseif wday == 4 then
		weekstr = "水"
	elseif wday == 5 then
		weekstr = "木"
	elseif wday == 6 then
		weekstr = "金"
	elseif wday == 7 then
		weekstr = "土"
	end
	local timestamp = string.format(" %2.2d:%2.2d(%s)",now["hour"],now["min"],weekstr)
	local msg = str .. timestamp
	-- メッセージ登録
	stat_check(set_shared_data(msg_ptr_pre .. node,""))
	stat_check(add_shared_strlist(ch_prefix .. node,msg,false,max_msg_list))
	local cntr = stat_check(count_shared_strlist(ch_prefix .. node))
	-- LCD表示
	mgcp_lcd_jprintln(node,tostring(cntr) .. "/" .. tostring(cntr) .. " " ..  msg,true,font,nil,nil,split,fnt_size,fg_color,bg_color)
end

-------------------------------------
-- メッセージ表示・コマンド
-------------------------------------
if p_data ~= "" then
	for i,v in ipairs(target_node_list) do
		if v.node_type == "M5Stack" then	-- M5Stack
			disp_msg_and_store_history(v.node_name,p_data,2)
			-- BUZZER 出力
			if use_buzzer then
				stat_check(set_shared_data(buzzer_flag_pre .. v.node_name ,"1"))
				script_fork_exec("MGCP/DEVICE/M5_STACK_BUZZER","node,state",v.node_name .. "," .. "blink")
			end
		end
		if v.node_type == "M5Stick" then	-- M5Stick-C
			disp_msg_and_store_history(v.node_name,p_data,1)
			-- LED 点滅
			if use_led_blink then
				stat_check(set_shared_data(blink_flag_pre .. v.node_name ,"1"))
				script_fork_exec("MGCP/DEVICE/M5_STICK_LED","node,state",v.node_name .. "," .. "blink")
			end
		end
	end
end

-- m5stack_node_list, m5stickc_node_list リストに含まれているノードにメッセージ表示
function disp_msg_only(str)
	for i,v in ipairs(target_node_list) do
		if v.node_type == "M5Stack" then	-- M5Stack
			mgcp_lcd_jprintln(v.node_name,str,true,font,nil,nil,split,2,fg_color,bg_color)
		end
		if v.node_type == "M5Stick" then	-- M5Stick-C
			mgcp_lcd_jprintln(v.node_name,str,true,font,nil,nil,split,1,fg_color,bg_color)
		end
	end
end

-------------------------------------
-- １つ後のメッセージ表示・コマンド
-------------------------------------
if g_params["next"] and (p_node ~= "") then
	-- ノードがLED点滅やブザー出力状態の時はクリアして過去メッセージ表示は次回コール時に行う
	if blink_mode_clear(p_node) then return end
	if buzzer_mode_clear(p_node) then return end
	-- メッセージリスト取得
	local msg_list = stat_check(get_shared_strlist(ch_prefix .. p_node))
	if #msg_list < 2 then return end
	-- メッセージリスト中の表示対象位置を決める
	local ptr = stat_check(get_shared_data(msg_ptr_pre .. p_node))
	local ptr_i
	if ptr == "" then
		ptr_i = 1
	else
		ptr_i = tonumber(ptr)
		ptr_i = ptr_i + 1
		if ptr_i > #msg_list then ptr_i = 1 end
	end
	stat_check(set_shared_data(msg_ptr_pre .. p_node,tostring(ptr_i)))
	-- メッセージ表示
	disp_msg_only(tostring(ptr_i) .. "/" .. tostring(#msg_list) .. " " .. msg_list[ptr_i])
end

-------------------------------------
-- １つ前のメッセージ表示・コマンド
-------------------------------------
if g_params["prev"] and (p_node ~= "") then
	-- 現在ノードがLED点滅状態の時は消灯のみで過去メッセージ表示は次回コール時に行う
	if blink_mode_clear(p_node) then return end
	if buzzer_mode_clear(p_node) then return end
	-- メッセージリスト取得
	local msg_list = stat_check(get_shared_strlist(ch_prefix .. p_node))
	if #msg_list < 2 then return end
	-- メッセージリスト中の表示対象位置を決める
	local ptr = stat_check(get_shared_data(msg_ptr_pre .. p_node))
	local ptr_i
	if ptr == "" then
		ptr_i = #msg_list - 1
	else
		ptr_i = tonumber(ptr)
		ptr_i = ptr_i - 1
		if ptr_i < 1 then ptr_i = #msg_list end
	end
	stat_check(set_shared_data(msg_ptr_pre .. p_node,tostring(ptr_i)))
	-- メッセージ表示
	disp_msg_only(tostring(ptr_i) .. "/" .. tostring(#msg_list) .. " " .. msg_list[ptr_i])
end
