
--[[

*********************************************************************************
MGCP リクエストコマンドを受信時のイベントハンドラスクリプト。
MQTT_PUBLISH イベントハンドラから別スレッドでこのスクリプトが呼び出されます。
*********************************************************************************

REQUEST スクリプト起動時に渡されるパラメータ
---------------------------------------------------------------------------------
キー値          値                                                      値の例
---------------------------------------------------------------------------------

endpoint		MGCP データを受信した MQTT エンドポイント
				ClientID または Title名が入る							"endpoint1"

sender			イベント送信元の MGCP デバイスノード名					"STICK001"

recipient		イベント送信時に指定された送信対象ノード名
				"_ALL_" または MGCP デバイスノード名が入る				"_ALL_"

msg_id			リクエスト時のトピックに指定された <msg_id> 文字列
				指定されていない場合は値は入らない						"REQ_1234"

data			リクエストコマンドデータ。JSON 形式の文字列が格納されている。

															{"command":"version"}
				

]]

--[[
*******************************************************************************************************************
MGCP リクエストコマンドが abs_agent が動作するコンピュータのノードに対して送信されたときの処理を記述する。

この時、トピック名 は /request/<リクエスト先ホスト名(このコンピュータ名)>/<リクエスト元ホスト名>[/<msg_id>] 
で、リクエストコマンド文字列が格納されたメッセージを受信する。

ペイロードに格納された JSON 文字列の "command" タグに指定された文字列でリクエストされたサービスを識別する。
サービスの処理に必要なオプションデータも JSON で記述されたペイロードに格納されている。

もし、リクエスト時のトピックに <msg_id> が指定されていた場合には、処理が完了したら、リプライメッセージを
ペイロードに格納して MQTT ブローカにメッセージを送信する。

リプライメッセージを送信するときに使用するトピック名は、リクエストコマンドのトピック名に指定されていた 
<msg_id> 文字列を使用して /reply/<リクエスト元ホスト名>/<リクエスト先ノード名(このコンピュータ名)>/<msg_id> とする。

req_json_tbl テーブルにはリクエストメッセージ(JSONフォーマット) を Lua テーブルに変換した
ものが格納されています。req_json_tbl.xxx 等でアクセスするとその値を取り出すことができます。
文字列や数値などは JSON で指定された型のままアクセスできます。例えば {"str":"data","x":100} の様な
リクエストパラメータが指定された場合、タグ x の値を数値で取り出す場合には req_json_tbl.x になります。
文字列形式で指定されたタグ str の値を文字列で取り出す場合は同様に req_json_tbl.str でアクセスできます。

コマンド処理が正常終了した場合には reply_json 文字列変数にリプライ JSON 文字列を代入します。このとき
必ず"result" タグ（値は "success")を含めるようにしてください。 {"result":"success"}

コマンド処理でエラーが発生した場合にも同様に reply_json 文字列変数にリプライ JSON 文字列を代入します。このとき
必ず"result" タグ（値は "fail")を含めるようにしてください。 {"result":"fail"} 任意で "error_text" タグに
エラーの詳細メッセージを含めることもできます。

何れの場合でもコマンド処理を行った場合には command_processed 変数に true を代入します。

このイベントハンドラは常に別スレッドで起動されますので、コマンド処理は平行して複数実行される可能性がある
点に留意してください。稀に実行されるコマンドで時間がかかる処理を行いたい場合には、別のスクリプトに処理を
分けてそのスクリプトを script_fork_exec() 関数で別スレッドで呼び出すようにします。

注意：リクエストを行うコンピュータ側ではこのコンピュータのノード名を事前に知っておく必要があります。
その為に、このコンピュータで少なくとも１回(または定期的に)、MGCP/SEND_HEARTBEAT スクリプトを実行してください。
これによって heartbeat イベントが全ノードに送信されて、自身のノードの存在を知らせることができます。
*******************************************************************************************************************
]]

local command_processed = false
local reply_json = ""
local req_json_tbl = g_json.decode(g_params["data"])
local reply_topic = ""
if g_params["msg_id"] ~= nil then
	reply_topic = "/reply/" .. g_params["sender"] .. "/" ..  g_params["recipient"] .. "/" .. g_params["msg_id"]
end

----------------------------
-- 指定コマンド実行
----------------------------
local params = {}
params["request"] = g_params["data"]
if req_json_tbl.command then
	local script = "MGCP/HANDLER/cmd_" .. req_json_tbl.command
	if script_exists(script) then
		local stat,result = script_exec2(script,params)
		if stat then
			command_processed = true
			reply_json = result["reply"]
		end
	end
end

---------------------------
-- リプライメッセージ送信
---------------------------
if g_params["msg_id"] ~= nil then
	if not command_processed then
		if req_json_tbl.command then
			reply_json = '{"result":"fail","error_text":"command not found"}'
		else
			reply_json = '{"result":"fail","error_text":"command not specified"}'
		end
	end
	-- リクエストを受信したMQTTエンドポイント経由でリプライを送信
	stat_check(mqtt_publish(g_params["endpoint"],reply_topic,reply_json,0))
end

