
--[[

●機能概要

MQTT ブローカを利用したリクエスト->リプライ方式でコマンド送信とリプライ受信を行います。

preload/012_MQTT_MGCP/MGCP_LIB.lua ファイル内で定義された mgcp_command() 関数を内部で
コールします。詳しい仕様は MGCP_LIB.lua ファイル内のコメントをご覧ください。


●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

node		リクエストコマンドの送信対象のノード名						"ESP32Node1"

request		リクエストコマンド(JSONフォーマット) 文字列
			ここで指定するコマンドは、送信対象ノード側で提供しているコマンドを指定する。

																		{"command":"version"}

noreply		リプライメッセージを受信しない。値は何を指定しても構わない  "1"
			(オプションパラメータ)

timeout		リプライメッセージ受信までのタイムアウト時間を指定(ms)		6000
			(オプションパラメータ)


●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

Stauts			処理が成功した場合には "OK" が入る
				指定したサービスが提供されていない等で、サーバーから結果を正常に
				受け取れなかった場合にはエラーメッセージが入る
																		"OK"

reply 			サーバーから返されたリプライメッセージ(JSON フォーマット)

									{"result":"success","error_text":"","version":"1.0.0"}

●備考

このスクリプトを使用する前に、abs_agent の設定ファイルに MQTTブローカへの
エンドポイントを作成しておきます。また、MQTT PUBLISH メッセージを受信した場合に
実行される MQTT_PUBLISH イベントハンドラ中に、リプライメッセージ
受信時の処理を記述しておきます。詳しくは、MQTT_PUBLISH イベントハンドラ中の
コメントアウトされた処理部分をご覧ください。

このスクリプトで使用している MQTT ブローカエンドポイントの設定例です。
abs_agent.xml 設定ファイル中の MQTT 設定に記述します。詳しいタグの説明は
abs_agent ユーザーマニュアルを参照してください。

<ClientID> と <BrokerHostName> は使用する MQTT ブローカに合わせて下さい。

    <MQTT>
      <AutoOnline type="boolean">True</AutoOnline>
      <KeepAliveTimer type="integer">60</KeepAliveTimer>
      <EndPointList>
        <Item>
          <Title>リモートデバイス処理</Title>
          <ClientID>abs9k:12345-raspi</ClientID>
          <BrokerHostName>192.168.100.14</BrokerHostName>
          <PortNumber>1883</PortNumber>
          <AutoSubscribeTopicList>/reply/#</AutoSubscribeTopicList>
          <AutoSubscribeQoSList>0</AutoSubscribeQoSList>
          <UserName/>
          <Password/>
          <WillTopic/>
          <WillMessage/>
          <WillQoS>0</WillQoS>
          <WillRetain>False</WillRetain>
          <RecvBuffInit>2000</RecvBuffInit>
          <DetailLog>True</DetailLog>
        </Item>
      </EndPointList>
    </MQTT>

●abs_agent の WebAPI からこのスクリプトを実行する例

動作環境例：

* abs_agent のIP アドレスが 192.168.100.15 で Webサーバーはデフォルトの 8080 ポートで動作中
* 予め abs_agent 側で認証または作成したセッショントークンは "1234"
* コマンド実行対象の MGCP デバイスのノード名は "M5_N001"
* 実行する MGCP コマンドは {"command":"version"}

上記の abs_agent へのリクエスト URL は

http://192.168.100.15:8080/command/json/script?session=1234&name=MGCP/MGCP_EXEC&node=M5_N001&noquote=1&request={%22command%22:%22version%22}

正常にコマンド処理されたときのリプライ文字列(実際のリプライ文字列に改行は含まれません)

{"Result":"Success","ErrorText":"","TaskID":"LUA04C7C91297E92C",
"ResultParams":{"Status":"OK","reply":{"result":"success","program":"MGCP_M5Stack_Pro","version":"v0.06","license":"unlimited"}}}


●変更履歴

2019/01/30  WebAPI 実行例説明を追加した

2018/07/30  mgcp_command() に対応して endpoint パラメータを自動で取得するように変更

2018/07/09  endpoint パラメータのデフォルト値をグローバル共有変数から取得できるようにした

2017/12/23	初版作成

abs_agent copyright(c) All Blue System

]]

local noreply = false
local timeout = 2000
local stat

-------------------------
-- パラメータチェック
-------------------------

if g_params["noreply"] then
    noreply = true
end

if g_params["timeout"] then
    timeout = tonumber(g_params["timeout"])
end

if not g_params["node"] then
	script_result(g_taskid,"Status","*ERROR* node parameter error")
	return
end

if not g_params["request"] then
	script_result(g_taskid,"Status","*ERROR* request parameter error")
	return
end

-------------------------
-- MGCP コマンド実行
-------------------------
local stat,reply = mgcp_command(g_params["node"],g_params["request"],noreply,timeout)
if stat then
	script_result(g_taskid,"Status","OK")
	if reply then script_result(g_taskid,"reply",reply) end
else
	script_result(g_taskid,"Status","*ERROR* mgcp_command() failed.")
end

