
file_id = "MQTT_KEEP_ALIVE_TIMER"

--[[

●機能概要

現在ソケット接続中でかつエラーが発生していない MQTT エンドポイントに対して MQTT PINGREQ メッセージを
送信します。ソケットエラーが発生している MQTTエンドポイントに関しては、ブローカに再接続を試みます。
ソケット接続中ではない MQTT エンドポイントに関してはこのイベントハンドラではなにも行いません。

このスクリプトでは再接続を試みたときに、再びソケットエラーが発生したエンドポイントについては
接続不能と判断して、以降は再接続を試みないようになります。もし、永久にソケットの再接続を試みたい場合には
下記のスクリプト部分を

------------------------------------------------
-- エラーが発生しているエンドポイントを再接続する
-------------------------------------------------
for key,val in ipairs(id) do
    if enable[key] and (not ready[key]) then        -- 現在ソケット接続中でエラーが発生しているエンドポイントを対象にする

以下の様に変更することで実現できます。

------------------------------------------------
-- エラーが発生しているエンドポイントを再接続する
-------------------------------------------------
for key,val in ipairs(id) do
    if (not enable[key]) or (enable[key] and (not ready[key])) then    -- 現在ソケット未接続またはエラー発生中のエンドポイントを対象にする

上記の変更を加えると mqtt_close() 関数がソケット未接続の場合にエラー（既にソケットが閉じているため) を検出しますが問題ありません。

●備考

このスクリプトに変更を加える場合には、スクリプトの実行は長くても数秒以内で必ず終了するようにしてください。
同時実行可能なスクリプトの数に制限があるため、他のスクリプトの実行開始が待たされる原因にもなります。

●変更履歴

2016/06/01  abs_agent 用に移植

2015/05/14  初版作成  for ABS-9000 DeviceServer


]]

-- log_msg("start..",file_id)

----------------------------------------------
-- 最新のエンドポイントリストを取得
----------------------------------------------
local stat, enable, ready, title, id, host, port, subscribe_topic_list, subscribe_qos_list,
    user, password, will_topic, will_message, will_qos, will_retain, recv_buff_size,log = mqtt_all_list()
if not stat then error() end

----------------------------------------------
-- PING メッセージをブローカに送信
----------------------------------------------
for key,val in ipairs(id) do
    ----------------------------------------------------------------------------
    -- 現在ソケット接続中でエラーが発生していないエンドポイントのみを対象にする
    ----------------------------------------------------------------------------
    if enable[key] and ready[key] then
        ---------------------------------------
        -- PINGREQ メッセージをブローカに送信
        ---------------------------------------
        mqtt_ping(id[key])
    end
end

------------------------------------------------------------
-- 先の PINGREQ 送信時にエラーを検出している場合があるので、
-- 再び最新のエンドポイントリストを取得
------------------------------------------------------------
stat, enable, ready, title, id, host, port, subscribe_topic_list, subscribe_qos_list,
    user, password, will_topic, will_message, will_qos, will_retain, recv_buff_size,log = mqtt_all_list()
if not stat then error() end

------------------------------------------------
-- エラーが発生しているエンドポイントを再接続する
-------------------------------------------------
for key,val in ipairs(id) do

    ----------------------------------------------------------------------
    -- 現在ソケット接続中でエラーが発生しているエンドポイントを対象にする
    ----------------------------------------------------------------------
    if enable[key] and (not ready[key]) then
        log_msg("*WARNING* re-starting the endpoint ClientID = " .. id[key],file_id)

        --------------------------------
        -- エンドポイントのソケット削除
        --------------------------------
        mqtt_close(id[key])

        -----------------------------
        -- Broker にソケット接続開始
        -----------------------------
        if mqtt_open(id[key]) then

            ------------------------------------
            -- Broker に CONNECT リクエスト送信
            ------------------------------------
            local cstat,connack = mqtt_connect(id[key])
            if cstat then
                if (connack == 0) then
                    if subscribe_topic_list[key] ~= "" then
                        -----------------------
                        -- デフォルトTopic購読
                        -----------------------
                        mqtt_subscribe(id[key])
                    end
                else
                    ----------------------------------------------------
                    -- Brokerから CONNECT を拒否された時はソケット削除
                    ----------------------------------------------------
                    log_msg("connack is not 0, closing socket " .. id[key],file_id)
                    mqtt_close(id[key])
                end
            end
        end
    end
end

