
file_id = "MQTT_PUBLISH"

--[[

******************************************************************************

一つのスクリプトの実行は長くても数秒以内で必ず終了するようにしてください。
処理に時間がかかると、イベント処理の終了を待つサーバー側でタイムアウトが発生します。

また、同時実行可能なスクリプトの数に制限があるため、他のスクリプトの実行開始が
待たされる原因にもなります。

頻繁には発生しないイベントで、処理時間がかかるスクリプトを実行したい場合は
スクリプトを別に作成して、このイベントハンドラ中から script_fork_exec() を使用して
別スレッドで実行することを検討してください。

******************************************************************************


MQTT_PUBLISH スクリプト起動時に渡される追加パラメータ

---------------------------------------------------------------------------------
キー値          値                                                      値の例
---------------------------------------------------------------------------------
ClientID        エンドポイントの ClientID 文字列                        "abs9k:2222-eagle"

Title           エンドポイントに設定されたタイトル文字列。
                タイトル文字列が設定されていない場合には、"" 空文字列
                が入ります                                              "センサーデバイス#1"

MessageType     MQTT プロトコルで定義されたメッセージタイプが入ります。 "3"
                PUBSLIH メッセージの場合には常に "3"が設定されます

MessageID       Brokerから送信するときに使用された MQTT メッセージID が
                入ります。(QoS = 1 または QoS = 2 の場合) 値は "1" から
                "65535" の整数値をとります。
                QoS = 0 の場合には常に "0" が設定されます。             "1234"

Dup             MQTT 固定ヘッダ中の Dup フラグの値が設定されます。
                "0" または "1" の値をとります。                         "0"

QoS             MQTT 固定ヘッダ中の QoS フラグの値が設定されます。
                "0", "1", "2" の何れかの値をとります。                  "0"

Retain          MQTT 固定ヘッダ中の Retain フラグの値が設定されます。
                "0" または "1" の値をとります。                         "0"

PublishTopic    MQTT ブローカから受信した PUBLISH メッセージ中の Topic
                文字列。                                                "センサー/ノード1"

PublishData     MQTT ブローカから受信した PUBLISH メッセージ中のペイロー
                ドデータ。
                バイナリデータを16進数文字列に変換したものが格納されます
                ペイロードデータに格納されたデータが UTF-8 文字列の場合
                には文字列コードのバイト列が格納されています。
                イベントハンドラ中でこれらの文字列データをデコードする処
                理がデフォルトで記述されていますので、UTF-8 文字列を扱う
                場合にはデコード後の変数を利用することができます。      "010203414243"


PublishDataで渡されたペイロードデータを解析して作成される文字列変数

PublishString   PublishData に格納されたペイロードデータ部分のサイズが 
                2048 Bytes以内の場合に、データバイト列を UTF-8形式で
                文字列にデコードした結果を PublishString に格納します。
                変換対象のバイト列のサイズを変更したいときには該当する
                スクリプト部分を変更して下さい。

]]

------------------------------------------------------------------------------------------
-- 受信したペイロードデータのサイズが 2048 bytes 以内の場合には
-- バイナリデータ列を UTF-8 文字列としてデコードしたものを PublishString 変数に格納する
------------------------------------------------------------------------------------------
local PublishString = ""
local pub_len = string.len(g_params["PublishData"]) / 2
if pub_len < 4096 then
    PublishString = readUTF_hex(bit_tohex(pub_len,4) .. g_params["PublishData"])
end
log_msg(g_params["Title"] .. "[" .. g_params["ClientID"] .. "] msg:" .. g_params["MessageID"] .. " dup:" .. g_params["Dup"]  .. 
" retain:" .. g_params["Retain"]  .. " qos:" .. g_params["QoS"]  .. " topic:" .. g_params["PublishTopic"]  .. " " .. PublishString,file_id)

---------------------------------------------------------------------------------------------------------------------
-- MGCPプロトコル処理用、使用しない場合にはコメントアウトできます。詳細は "MGCP ユーザーマニュアル" を参照して下さい
---------------------------------------------------------------------------------------------------------------------
local topic_arr = topic_to_tbl(g_params["PublishTopic"])
if #topic_arr >= 3 then
	local endpoint = g_params["Title"]
	if endpoint == "" then endpoint = g_params["ClientID"] end
	local params = {}
	params["endpoint"] = endpoint
	params["sender"] = topic_arr[3]
	params["recipient"] = topic_arr[2]
	params["data"] = PublishString
	-- request packet
	if (topic_arr[1] == "request") then
		if topic_arr[4] then params["msg_id"] = topic_arr[4] end
		script_fork_exec("MGCP/HANDLER/REQUEST",params)
	end
	-- event packet
	if (topic_arr[1] == "event") and (#topic_arr == 5) then
		params["device"] = string.match(topic_arr[4],"MGCP_(.+)_.+")
		params["type"] = topic_arr[5]
		script_fork_exec("MGCP/HANDLER/EVENT",params)
	end
	-- reply packet
--[[
	このコンピュータから MGCPリクエストコマンド送信後に、MQTT ブローカ経由で受信したリプライメッセージ処理の記述。
	リプライメッセージのトピックは /reply/<リクエスト元ホスト名(このコンピュータ名)>/<リクエスト先ノード名>/<msg_id> 
	で、<msg_id> はリクエスト時に送信した <msg_id>と同じ文字列になっている。
	リクエスト時に設定した <msg_id> の名前のグローバル共有変数を、このコンピュータからリクエストを行ったかどうかを
	判断するフラグとして使用する。また同時に、複数のサーバーからリプライを受信した場合に最初に受信したリプライのみを
	選択するためのフラグとしても利用する。MQTT ブローカ経由で受信したサーバーのリプライメッセージは、
	アンダースコア文字を追加した "_<msg_id>" の名前のグローバル共有変数にセットする。
	その後、イベントをセット状態にしてリクエスト発行側で待ち状態になっているタスクを復帰させる。
]]
	if (topic_arr[1] == "reply") and topic_arr[4] then
		local msg_id = topic_arr[4]
		local stat,val = get_shared_data(msg_id,true)
		if val == "1" then 
			set_shared_data("_" .. msg_id,PublishString,true) 
			event_set2x(msg_id)
		end
	end
end

----------------------------------------------------------
--- ユーザー環境用 MQTT PUBLISH イベント処理はここから ---
----------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------
-- トピック名が "/<sender_hostname>/switch" でメッセージ内容が "ON" の場合に LED 点灯、それ以外の場合には消灯させる
-- 下記のサンプルスクリプトの説明は abs_agent ユーザーマニュアルをご覧ください
----------------------------------------------------------------------------------------------------------------------
--[[
if string.match(g_params["PublishTopic"],"/.+/(.+)") == "switch" then
	if PublishString == "ON" then
		if not raspi_gpio_write(18,true) then error() end
	else
		if not raspi_gpio_write(18,false) then error() end
	end
end
]]

----------------------------------------------------------------------------------------------------------------------
-- MQTTアラーム装置のメッセージ解析
-- 下記のサンプルスクリプトの説明は、ホームページの Blog記事 "MQTT接続のアラーム装置作成" をご覧ください
----------------------------------------------------------------------------------------------------------------------
--[[
local flag,sender = string.match(g_params["PublishTopic"],"/alarm/(.+)/(.+)")
if flag == "switch" then
	------------------------------------------------------------------------------------------------------------------
	-- トピックが "/alarm/switch/<送信側ホスト名または任意の文字列>" の場合にメッセージを JSON デコードして、
	-- "sw1" キーの値が "on" の場合にはグローバル共有変数 alarm_red, alarm_yellow, alarm_green, alarm_buzz を削除する
	------------------------------------------------------------------------------------------------------------------
	local msg = g_json.decode(PublishString)
	if msg.sw1 == "on" then
		set_shared_data("alarm_red","")
		set_shared_data("alarm_yellow","")
		set_shared_data("alarm_green","")
		set_shared_data("alarm_blue","")
		set_shared_data("alarm_buzz","")
	end
end

if flag == "signal" then
	---------------------------------------------------------------------------------------------------------
	-- トピックが "/alarm/signal/<送信側ホスト名または任意の文字列>" の場合にメッセージを JSON デコードして、
	-- 得られたオブジェクトのキーと値をそのままグローバル共有変数に設定する。
	---------------------------------------------------------------------------------------------------------
	local msg = g_json.decode(PublishString)
	for k,v in pairs(msg) do
		set_shared_data(k,v)
	end
end
]]

-----------------------------------------------------------------------------------------------
-- MQTT-PUBLISH で配信されたデータを後から、スクリプトやWebAPI、EXCEL等から取得できるようにする
-- MQTT Topic 名ごとに文字列リスト(チャンネル)を分けて、ペイロード中の文字列をリストに格納する
-- add_shared_strlist() の第４パラメータは、トピック毎(文字列リスト毎)に保存する最大データ数
-----------------------------------------------------------------------------------------------
--[[
if PublishString ~= "" then
    if not add_shared_strlist(g_params["PublishTopic"],PublishString,false,100) then error() end
end 
]]
