
--[[

●機能概要

グローバル共有変数にセットされたアラーム状態を示すフラグ値に対応して、
Raspberry Pi の GPIO に接続された LEDの点滅やブザーを鳴らす。

●参照するグローバル共有変数

---------------------------------------------------------------------------------
キー値            値                            　 操作するGPIO# と設定値                
---------------------------------------------------------------------------------
"alarm_red"        "off" or "" (変数未定義)    GPIO#4  Low                
                "on"                           GPIO#4  High
                "blink"                        GPIO#4  Low-High 繰り返し

"alarm_yellow"    "off" or "" (変数未定義)     GPIO#17 Low                
                "on"                           GPIO#17 High
                "blink"                        GPIO#17 Low-High 繰り返し

"alarm_green"   "off" or "" (変数未定義)       GPIO#18  Low                
                "on"                           GPIO#18  High
                "blink"                        GPIO#18  Low-High 繰り返し

"alarm_buzz"    "off" or "" (変数未定義)       GPIO#27  Low                
                "beep1"                        GPIO#27  Low-High 早い繰り返し
                "beep2"                        GPIO#27  Low-High 遅い繰り返し


●備考

このスクリプトは無限ループに入って終了しないので、必ず別スレッドで起動してください。

●変更履歴

2017/01/22    log_msg() コール時の file_id 変数を g_script に修正

2016/08/23    初版作成

copyright(c) All Blue System

]]

-- ２重起動防止用チェック
if not exclusive_check(g_script) then
    log_msg("*ERROR* exclusive_check() failed. script = " .. g_script,g_script)
    return
end
log_msg("start..   TaskID = " .. g_taskid,g_script)

-- アラーム状態出力用の Raspberry Pi GPIO ピン番号設定
local RED_pin = 4
local YELLOW_pin = 17
local GREEN_pin = 18
local BUZZ_pin = 27
local SW_pin = 23
local short_timer_interval = 80      -- beep1 ブザー音間隔(ms)
local long_timer_repeat_count = 8    -- beep2 ブザー音間隔、LED 点滅間隔、グローバル共有変数のチェック間隔
                                      -- short_timer_interval の繰り返し回数で指定する

-- グローバル共有変数の最新の値のキャッシュ, beep2 間隔でグローバル共有変数から最新の値をロードして更新される
local alarm_red = ""
local alarm_yellow = ""
local alarm_green = ""
local alarm_buzz = ""

local led_stat = false            -- LEDとブザー(beep2) High - Low 繰り返しの現在のステート値
local buzz_stat = false           -- ブザー(beep1) High - Low 繰り返しの現在のステート値

-- LED と スイッチを接続する GPIO のモードを初期設定する。
if not raspi_gpio_config(RED_pin,"output","off") then error() end 
if not raspi_gpio_config(YELLOW_pin,"output","off") then error() end 
if not raspi_gpio_config(GREEN_pin,"output","off") then error() end 
if not raspi_gpio_config(BUZZ_pin,"output","off") then error() end 
if not raspi_gpio_config(SW_pin,"input","pullup") then error() end

-- スイッチ入力を検知するために RASPI_CHANGE_DETECT イベントを有効にする
if not raspi_change_detect(SW_pin,true) then error() end

-- short_timer_interval 間隔で実行
function short_cyc_task()

    -- ブザー(beep1) の状態を反転する
    buzz_stat = not buzz_stat

    -- ブザー(beep1)の GPIO 出力
    if alarm_buzz == "beep1" then
        if not raspi_gpio_write(BUZZ_pin,buzz_stat) then error() end 
    elseif (alarm_buzz == "") or (alarm_buzz == "off") then
        if not raspi_gpio_write(BUZZ_pin,false) then error() end 
    end

end

-- short_timer_interval * long_timer_repeat_count 間隔で実行
function long_cyc_task()
    local stat

    -- 最新のグローバル共有変数の値をキャッシュにロード 
    stat,alarm_red = get_shared_data("alarm_red")
    stat,alarm_yellow = get_shared_data("alarm_yellow")
    stat,alarm_green = get_shared_data("alarm_green")
    stat,alarm_buzz = get_shared_data("alarm_buzz")
 
    -- LED の状態を反転する
    led_stat = not led_stat

    -- LED とブザー(beep2)の GPIO 出力
    if alarm_red == "on" then
        if not raspi_gpio_write(RED_pin,true) then error() end 
    elseif alarm_red == "blink" then
        if not raspi_gpio_write(RED_pin,led_stat) then error() end 
    else
        if not raspi_gpio_write(RED_pin,false) then error() end 
    end

    if alarm_yellow == "on" then
        if not raspi_gpio_write(YELLOW_pin,true) then error() end 
    elseif alarm_yellow == "blink" then
        if not raspi_gpio_write(YELLOW_pin,led_stat) then error() end 
    else
        if not raspi_gpio_write(YELLOW_pin,false) then error() end 
    end

    if alarm_green == "on" then
        if not raspi_gpio_write(GREEN_pin,true) then error() end 
    elseif alarm_green == "blink" then
        if not raspi_gpio_write(GREEN_pin,led_stat) then error() end 
    else
        if not raspi_gpio_write(GREEN_pin,false) then error() end 
    end

    if alarm_buzz == "beep2" then
        if not raspi_gpio_write(BUZZ_pin,led_stat) then error() end 
    elseif (alarm_buzz == "") or (alarm_buzz == "off") then
        if not raspi_gpio_write(BUZZ_pin,false) then error() end 
    end

end

-- メインタスク、グローバル変数の値を定期的に監視してアラーム出力を行う
local cntr_long_timer = 0
while true do -- 無限ループを停止させる場合には script_kill() または "agent_task -k <taskid>" コマンドを使用する
    wait_time(short_timer_interval)
    if cntr_long_timer >= (long_timer_repeat_count - 1) then
        cntr_long_timer = 0
        long_cyc_task() -- 約 640ms 間隔で実行
    else
        cntr_long_timer = cntr_long_timer + 1
    end
    short_cyc_task() -- 約 80ms 間隔で実行
end

