
--[[

●機能概要

I2C バスに接続した気圧・温度・湿度センサー(BME280) の値を取得する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------
bus             I2C バス番号                                     "1"
                "0" または "1"を指定、省略時は "1" を使用する

init			このパラメータを指定した場合には、強制的にデバイス
				初期化と補償データレジスタの取得を行う。
				パラメータ値は任意。		 					 "1"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------

temperature     センサーから取得した摂氏温度                     "12.5"
                                                                 "-25.0"

humidity		センサーから取得した相対湿度(%)                  "75.0"

pressure	    センサーから取得した気圧(hPa)	                 "1013.1"


●備考


●使用条件・免責事項

このスクリプトファイルは自由にお客様が複製、改変を行うことができます。またお客様のアプリケーションと
共にこのファイルを配布することができます。

このスクリプトは公開されているデバイス仕様書を元に、オールブルーシステムがサンプル実装したものです。
お客様がこのファイルを利用される場合には、お客様または第三者に損害が生じた場合でも、
オールブルーシステムは損害賠償その他一切の責任を負担しません。


●変更履歴

2017/03/06    初版作成

abs_agent	copyright(c) All Blue System


]]

local slave_addr = "76"
local bus = 1
local stat,data,result

-------------------------
-- パラメータチェック
-------------------------
if g_params["bus"] then
    bus = tonumber(g_params["bus"])
end

-----------------------------------------------------------------------
-- 初期化と補償データレジスタの取得
-----------------------------------------------------------------------
stat,data = get_shared_data("BME280_REGVAL")

if (data == "") or g_params["init"] then
	log_msg("initialize and setup the device",g_script)

	-----------------------------------------------------------------------
	-- BME280 の測定パラメータ設定
	-----------------------------------------------------------------------
	local osrs_t = 1			-- Temperature oversampling x 1
	local osrs_p = 1			-- Pressure oversampling x 1
	local osrs_h = 1			-- Humidity oversampling x 1
	local mode   = 3			-- Normal mode
	local t_sb   = 5			-- Tstandby 1000ms
	local filter = 0			-- Filter off
	local spi3w_en = 0			-- wire SPI Disable

	local ctrl_meas_reg = bit_or(bit_lshift(osrs_t,5),bit_lshift(osrs_p,2),mode)
	local config_reg    = bit_or(bit_lshift(t_sb,5),bit_lshift(filter,2),spi3w_en)
	local ctrl_hum_reg  = osrs_h

	if not raspi_i2c_write(bus,slave_addr,"F2" .. bit_tohex(ctrl_hum_reg,2)) then error() end
	if not raspi_i2c_write(bus,slave_addr,"F4" .. bit_tohex(ctrl_meas_reg,2)) then error() end
	if not raspi_i2c_write(bus,slave_addr,"F5" .. bit_tohex(config_reg,2)) then error() end

	---------------------------------------------------------------------------
	-- BME280 補償データレジスタの値を取得後共有データに保存
	-- 次回からのスクリプト実行時には共有データに保存されたレジスタ値を使用
	---------------------------------------------------------------------------
	stat = raspi_i2c_write(bus,slave_addr,"88")
	if not stat then error() end
	stat,result = raspi_i2c_read(bus,slave_addr,24)
	if not stat then error() end
	data = result
	
	stat,result = raspi_i2c_write(bus,slave_addr,"A1",1)
	if not stat then error() end
	data = data .. result
	
	stat,result = raspi_i2c_write(bus,slave_addr,"E1",7)
	if not stat then error() end
	data = data .. result

	set_shared_data("BME280_REGVAL",data)

end

------------------------------------
-- 補償パラメータ計算
------------------------------------
local reg = hex_to_tbl(data)
local dig_T1 = reg[1] + bit_lshift(reg[2],8)
local dig_T2 = bit_tosigned(reg[3] + bit_lshift(reg[4],8),16)
local dig_T3 = bit_tosigned(reg[5] + bit_lshift(reg[6],8),16)
local dig_P1 = reg[7] + bit_lshift(reg[8],8)
local dig_P2 = bit_tosigned(reg[9] + bit_lshift(reg[10],8),16)
local dig_P3 = bit_tosigned(reg[11] + bit_lshift(reg[12],8),16)
local dig_P4 = bit_tosigned(reg[13] + bit_lshift(reg[14],8),16)
local dig_P5 = bit_tosigned(reg[15] + bit_lshift(reg[16],8),16)
local dig_P6 = bit_tosigned(reg[17] + bit_lshift(reg[18],8),16)
local dig_P7 = bit_tosigned(reg[19] + bit_lshift(reg[20],8),16)
local dig_P8 = bit_tosigned(reg[21] + bit_lshift(reg[22],8),16)
local dig_P9 = bit_tosigned(reg[23] + bit_lshift(reg[24],8),16)
local dig_H1 = reg[25]
local dig_H2 = bit_tosigned(reg[26] + bit_lshift(reg[27],8),16)
local dig_H3 = reg[28]
local dig_H4 = bit_tosigned(bit_and(reg[30],0x0f) + bit_lshift(reg[29],4),16)
local dig_H5 = bit_tosigned(bit_lshift(reg[31],4) + bit_and(bit_rshift(reg[30],4),0x0f),16)
local dig_H6 = bit_tosigned(reg[32],8)

local t_fine -- compensate_T() 関数内で値をセットするので、値を参照する時には compensate_T() を先にコールしておくこと

function compensate_T(adc_T)
	local v1 = (adc_T / 16384.0 - dig_T1 / 1024.0) * dig_T2
	local v2 = (adc_T / 131072.0 - dig_T1 / 8192.0) * (adc_T / 131072.0 - dig_T1 / 8192.0) * dig_T3
	t_fine = v1 + v2
	local temperature = t_fine / 5120.0
	return temperature
end

function compensate_P(adc_P)
	local pressure = 0.0
	local v1 = (t_fine / 2.0) - 64000.0
	local v2 = (((v1 / 4.0) * (v1 / 4.0)) / 2048) * dig_P6
	v2 = v2 + ((v1 * dig_P5) * 2.0)
	v2 = (v2 / 4.0) + (dig_P4 * 65536.0)
	v1 = (((dig_P3 * (((v1 / 4.0) * (v1 / 4.0)) / 8192)) / 8)  + ((dig_P2 * v1) / 2.0)) / 262144
	v1 = ((32768 + v1) * dig_P1) / 32768

	local pressure = ((1048576 - adc_P) - (v2 / 4096)) * 3125
	if pressure < 0x80000000 then
		pressure = (pressure * 2.0) / v1
	else
		pressure = (pressure / v1) * 2
	end
	v1 = (dig_P9 * (((pressure / 8.0) * (pressure / 8.0)) / 8192.0)) / 4096
	v2 = ((pressure / 4.0) * dig_P8) / 8192.0
	pressure = pressure + ((v1 + v2 + dig_P7) / 16.0)  
	
	return pressure/100
end

function compensate_H(adc_H)
	local var_h = t_fine - 76800.0
	var_h = (adc_H - (dig_H4 * 64.0 + dig_H5/16384.0 * var_h)) * (dig_H2 / 65536.0 * (1.0 + dig_H6 / 67108864.0 * var_h * (1.0 + dig_H3 / 67108864.0 * var_h)))
	var_h = var_h * (1.0 - dig_H1 * var_h / 524288.0)
	if var_h > 100.0 then
		var_h = 100.0
	end
	if var_h < 0.0 then
		var_h = 0.0
	end
	return var_h
end

------------------------------------
-- 現在の測定値取得
------------------------------------
stat,result = raspi_i2c_write(bus,slave_addr,"F7",8)
if not stat then error() end
local raw = hex_to_tbl(result)

local pres_raw = bit_or(bit_lshift(raw[1],12),bit_lshift(raw[2],4),bit_rshift(raw[3],4))
local temp_raw = bit_or(bit_lshift(raw[4],12),bit_lshift(raw[5],4),bit_rshift(raw[6],4))
local hum_raw  = bit_or(bit_lshift(raw[7],8),raw[8])
	
local temp = compensate_T(temp_raw)
local press =compensate_P(pres_raw)
local humidity =compensate_H(hum_raw)

script_result(g_taskid,"temperature",string.format("%5.1f",temp))
script_result(g_taskid,"pressure",string.format("%7.1f",press))
script_result(g_taskid,"humidity",string.format("%5.1f",humidity))
