
--[[

●機能概要

環境センサーボード(スイッチサイエンス社製)に搭載している BME280 センサと光センサの
測定値を AQM1248A グラフィックLCD に表示する。


●参照するグローバル共有変数

---------------------------------------------------------------------------------
キー値                      値              
---------------------------------------------------------------------------------
ENVSENSOR_ACTIVE_PAGE    "1" .. "4"

LCD に表示するページを切り替える。
センサーボードのスイッチ入力に対応したイベントハンドラを設定してこの値を変更すると、
LCD に表示するページを切り替えることができる。


●備考

このスクリプトは無限ループに入って終了しないので、必ず別スレッドで起動してください。

●変更履歴

2022/6/11      スクリプト修正、その他改良

2017/03/11     初版作成

copyright(c) 2017-2022 All Blue System

]]

-- ２重起動防止用チェック
if not exclusive_check(g_script) then
    log_msg("*ERROR* exclusive_check() failed. script = " .. g_script,file_id)
    return
end
log_msg("start..   TaskID = " .. g_taskid,g_script)

-- 表示ページ切り替えスイッチ(SW1) の GPIO モード設定
-- スイッチ入力時のRASPI_CHANGE_DETECT イベントハンドラで、表示ページ切り替え用のグローバル共有変数を更新すること
local SW_pin = 24
if not raspi_gpio_config(SW_pin,"input","off") then error() end
if not raspi_change_detect(SW_pin,true) then error() end

-- デバイスとタスク内変数初期化

raspi_AQM1248A_try_init()							-- LCD AQM1284A 初期化

local active_page_var = "ENVSENSOR_ACTIVE_PAGE"	-- LCD 表示ページ切り替え用のグローバル共有変数

local run_indicator = true							-- 画面更新 Indicator
local ip_address = ""								-- 取得した自身の IP アドレスを表示するときに使用

---------------------------------------------------------------------
-- BME280 センサー
---------------------------------------------------------------------
function BME280_page()
	local BME280 = stat_check(script_exec2("RASPI/DEVICE/BME280_READ","",""))
	graphic_clear()
	graphic_print(22,1,BME280["temperature"] .. "[ ]",2)
	graphic_draw_bitmap(80,32,8,8,list_to_hex(0x02,0x05,0x02,0x3c,0x42,0x42,0x24,0x00),true,true,2) -- "℃" 文字
	graphic_print(22,3,BME280["humidity"] .. "[%]",2)
	graphic_print(6,5,BME280["pressure"] .. "[hP]",2)
	if run_indicator then
		graphic_draw_circle(120,38,6,true,true) 
	end
	raspi_AQM1248A_display()
end

---------------------------------------------------------------------
-- 光センサー
---------------------------------------------------------------------
function LightSensor_page()
	local ENVSENSOR = stat_check(script_exec2("RASPI/DEVICE/ENVSENSOR_LIGHT_READ","",""))
	graphic_clear()
	graphic_print(10,2,ENVSENSOR["light"],3)
	graphic_print(70,2,"[Lu]",2)
	local bright = math.ceil(110 * tonumber(ENVSENSOR["light"]) / 255)
	graphic_draw_rect(10,5,110,15,true,false)
	graphic_draw_rect(10,5,bright,15,true,true)
	raspi_AQM1248A_display()
end

---------------------------------------------------------------------
-- デジタル時計
---------------------------------------------------------------------
function clock_page()
	local now = os.date "*t"
	local wday = stat_check(day_of_week(now["year"],now["month"],now["day"]))
	local weekstr
	if wday == 1 then 
		weekstr = "Sun"
	elseif wday == 2 then
		weekstr = "Mon"
	elseif wday == 3 then
		weekstr = "Tue"
	elseif wday == 4 then
		weekstr = "Wed"
	elseif wday == 5 then
		weekstr = "Thu"
	elseif wday == 6 then
		weekstr = "Fri"
	elseif wday == 7 then
		weekstr = "Sat"
	end;
	local data1 = string.format("%2.2d/%2.2d %s",now["month"],now["day"],weekstr)
	local data2 = ""
	if run_indicator then
		data2 = string.format("%2.2d:%2.2d",now["hour"],now["min"])
	else
		data2 = string.format("%2.2d %2.2d",now["hour"],now["min"])
	end
	graphic_clear()
	graphic_print(15,1,data1,2)
	graphic_print(20,5,data2,3)
	raspi_AQM1248A_display()
end

---------------------------------------------------------------------
-- IPアドレスと時計
---------------------------------------------------------------------
function ip_page()
	if ip_address == "" then
		local info = stat_check(script_exec2("OS/GET_IP","",""))
		ip_address = info["IPAddress"]
	end
	local now = os.date "*t"
	local data1 = ""
	if run_indicator then
		data1 = string.format("%2.2d:%2.2d",now["hour"],now["min"])
	else
		data1 = string.format("%2.2d %2.2d",now["hour"],now["min"])
	end
	graphic_clear()
	graphic_print(35,1,data1,2)
	graphic_print(0,3,"IP=" .. ip_address,2)
	raspi_AQM1248A_display()
end

-- 指定したグローバル共有変数の値が変更されるか、もしくは指定されたカウント値 x 10ms 経過するまで内部でウェイト
function global_change_wait(global_name,start_val,max_wait_cntr)
	local stat,new_val
	local cntr = 0
	repeat
		wait_time(10)
		cntr = cntr + 1
		stat,new_val = get_shared_data(global_name)
	until (cntr >= max_wait_cntr) or (start_val ~= new_val)
end

-----------------------------------------------------------------------------------------------------------------
-- メインループ。無限ループを停止させる場合には script_kill() または "agent_task -k <taskid>" コマンドを使用する
-----------------------------------------------------------------------------------------------------------------

set_shared_data(active_page_var,"1")	-- 初期表示ページ設定
while true do
	local update_interval				-- 表示更新間隔の初期値(10msで割った数を指定)
	run_indicator = not run_indicator
	local page = stat_check(get_shared_data(active_page_var))
	if page == "1" then
		BME280_page()
		update_interval = 100
	elseif page == "2" then
		LightSensor_page()
		update_interval = 30
	elseif page == "3" then
		clock_page()
		ip_address = "" -- ページ切り替え時に ip_page() 内で IP アドレスを再取得させる
		update_interval = 50
	elseif page == "4" then
		ip_page()
		update_interval = 50
	end
	global_change_wait(active_page_var,page,update_interval)
end

