
--[[

●機能概要

FASTDB に保存したデータの時系列グラフを LCDデバイスに描画する

●制限事項

このスクリプトでは graphic2 ライブラリを使用して LCD デバイスに出力を行います。
デフォルトでは Raspberry Pi に接続した下記のデバイスに対応しています。

* AQM1248A(128x48)
* OLED1306(128x64)
* ST7789 Color TFT (240x240)
* Pimoroni製 Display HAT Mini LCD (ST7789 320x240)

これら以外のデバイスに出力を行う場合には、予めデバイス用の preload ライブラリを
作成した後、このスクリプト中の "device" パラメータ処理部分にコードを追加して下さい。 

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

key				(必須)集計対象のデータが格納された FASTDB キー名

range			グラフの集計範囲を指定する。下記の値が有効で省略時は "day" を設定する。

				"6h"	直近6時間
				"day"	当日(24時間)
				"2d"	昨日と今日の二日間
				"week"	直近1週間

title			グラフタイトル文字列。ASCII4文字 or 日本語2文字程度で指定
				省略時または空文字列指定時はタイトルと集計範囲をチャートに表示しない。

round			Y軸ラベルの数字をどれぐらいの精度で丸めるかを決める。省略時は自動で決定する。

y_min			Y軸の最低値を指定する。省略時は自動で決定する。

y_max			Y軸の最大値を指定する。省略時は自動で決定する。

device			LCDデバイス種別を指定する。下記の値が有効で省略時は "1" を設定する。

				"1"		AQM1248A(128x48)
				"2"		OLED1306(128x64)
				"3"		ST7789 Color TFT (240x240)
				"4"		Pimoroni製 Display HAT Mini LCD (ST7789 320x240)

grid			グラフ中の格子目盛の表示指定。省略時は"1"または"2"を自動設定する
				"0"		格子目盛を表示しない
				"1"		ドットで表示
				"2"		十字で表示

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

Status			実行結果が正常の場合には "OK" を返す。エラー発生時はエラーメッセージを格納する


●備考

device に指定する LCDデバイス用の preload ライブラリを予め有効にして下さい。
設定時は <abs_agent インストールディレクトリ>/scripts/preload/100_RASPI/ ディレクトリ内にある
デバイス毎のライブラリディレクトリ名を、先頭の "_" を取り除いてからabs_agent を再起動させます。

このスクリプトではグローバル共有メモリエリアにインストールされた、"ビットマップフォント"と、
"ディスプレイバッファ"にアクセスします。このとき、abs_agent 起動後に最初の１回目の実行時は
スクリプト内からロードと初期化動作を行うため、数秒余分に実行時間がかかります。

●制限事項

Y軸ラベルが小数点やマイナス符号を持っている場合に、Y軸ラベルの文字列が X軸右側にはみ出ることがあります。
必要に応じて使用するデバイス用に、グラフの原点をずらすなどのスクリプト修正を行ってください。


●変更履歴

2023/9/25	Pimoroni製 Display HAT Mini LCD(ST7789 320x240) デバイスに対応

2023/2/22   集計結果の有効レコード数が 0 の場合に、LCD にエラーメッセージを表示するようにした

2022/11/2	初版作成

abs_agent	copyright(c) 2022 All Blue System

*******************************************************************************************
]]

---------------------------------
-- スクリプトパラメータ
---------------------------------
local fastdb_key = ""
if g_params["key"] then
	fastdb_key = g_params["key"]
else
	error()
end
local title = g_params["title"] or ""
local remote = g_params["remote"] or ""

--------------------------------------------------
-- デバイスタイプに対応した表示パラメータ設定
--------------------------------------------------
local device =  g_params["device"] or "1"
local title_font = "SHINONOME12_FONTX"					-- 日本語(12x12) ASCII(6x12) 
local tick_font = "ASCII58_FONTX"						-- ASCII(5x8) 大文字のみ
local mini_font = "MISAKI_FONTX"						-- 日本語&ASCII(8x8) 
local font_size = 1									-- フォント拡大倍率(x1固定)
local x_mag = 1										-- チャート水平方向の拡大倍率
local lcd_ch,lcd_width,lcd_height,lcd_init,lcd_disp	-- LCDデバイスパラメータ・LCD操作関数
local c_line = 1										-- ライン描画色
local c_title = 1										-- タイトル描画色
local c_axis = 1										-- 軸描画色
local c_label = 1										-- ラベル描画色
local c_grid = 1										-- グリッド描画色
local grid												-- グリッド(格子目盛)の表示種別

if device == "1" then
	lcd_ch = AQM1248A_DISP_BUFF_CH
	lcd_width = AQM1248A_WIDTH		-- 128
	lcd_height = AQM1248A_HEIGHT	-- 48
	lcd_init = AQM1248A_try_init
	lcd_disp = AQM1248A_display
	grid =  g_params["grid"] or "1"
elseif device == "2" then
	lcd_ch = OLED1306_DISP_BUFF_CH
	lcd_width = OLED1306_WIDTH		-- 128
	lcd_height = OLED1306_HEIGHT	-- 64
	lcd_init = OLED1306_try_init
	lcd_disp = OLED1306_display
	grid =  g_params["grid"] or "1"
elseif device == "3" then
	lcd_ch = ST7789_DISP_BUFF_CH
	lcd_width = ST7789_WIDTH		-- 240
	lcd_height = ST7789_HEIGHT		-- 240
	lcd_init = ST7789_try_init
	lcd_disp = ST7789_display
	c_line = rgb565(255,255,0)
	c_title = rgb565(0xff,0xff,0xff)
	c_axis = rgb565(175,238,238)
	c_label = rgb565(175,238,238)
	c_grid = rgb565(30,144,255)
	title_font = "SHINONOME_FONTX"	-- 日本語(16x16) ASCII(16x8)
	tick_font = "SHINONOME14_FONTX"	-- 日本語(14x14) ASCII(14x7)
	mini_font = "SHINONOME14_FONTX"	-- 日本語(14x14) ASCII(14x7)
	x_mag = 2
	grid =  g_params["grid"] or "2"
elseif device == "4" then
	lcd_ch = DISPHATMINI_DISP_BUFF_CH
	lcd_width = DISPHATMINI_WIDTH		-- 320
	lcd_height = DISPHATMINI_HEIGHT		-- 240
	lcd_init = DISPHATMINI_try_init
	lcd_disp = DISPHATMINI_display
	c_line = rgb565(255,255,0)
	c_title = rgb565(0xff,0xff,0xff)
	c_axis = rgb565(175,238,238)
	c_label = rgb565(175,238,238)
	c_grid = rgb565(30,144,255)
	title_font = "AYU_FONTX"	-- 日本語(20x20) ASCII(20x10)
	tick_font = "SHINONOME14_FONTX"	-- 日本語(14x14) ASCII(14x7)
	mini_font = "SHINONOME14_FONTX"	-- 日本語(14x14) ASCII(14x7)
	x_mag = 2
	grid =  g_params["grid"] or "2"
else
	script_result("Status","*ERROR* unknown device = " .. tostring(device))
	return
end
if not lcd_disp then
	script_result("Status","*ERROR* device not initialized, device = " .. tostring(device))
	return
end

---------------------------------
-- LCDデバイス初期化
---------------------------------
fontx_try_init()
lcd_init()
graphic2_clear(lcd_ch)

if g_params["clear"] then
	lcd_disp(0)
	return
end

--------------------------------------------
-- range 毎の集計パラメータとタイトル設定
--------------------------------------------
local range = g_params["range"] or "day"
local sum_params = {}
local chart_head = title
local chart_head2 = ""

if range == "6h" then
	chart_head2 = "6時間"
	sum_params["Range"] = range
	sum_params["KeyList"] = fastdb_key
	sum_params["Interval"] = "300" -- 5分間隔で集計して 6hr/5min = 72レコード取得する
	--sum_params["TargetTime"] = "09:0:00" -- 試験用
	if device == "4" then x_mag = 3 end
elseif range == "day" then
	chart_head2 = "今日"
	sum_params["Range"] = range
	sum_params["KeyList"] = fastdb_key
	sum_params["Interval"] = "900" -- 15分間隔で集計して 24hr/15min = 96レコード取得する
elseif range == "2d" then
	chart_head2 = "2日"
	sum_params["Range"] = range
	sum_params["KeyList"] = fastdb_key
	sum_params["Interval"] = "1800" -- 30分間隔で集計して 48hr/30min = 96レコード取得する
elseif range == "week" then
	chart_head2 = "週間"
	sum_params["Range"] = range
	sum_params["KeyList"] = fastdb_key
	sum_params["Interval"] = "7200" -- 2時間間隔で集計して 7days/2hr = 84レコード取得する
	if device == "4" then x_mag = 3 end
else
	script_result("Status","*ERROR* unknown range, " .. range)
	return
end

---------------------------------
-- 集計計算
---------------------------------
local stat,summary_result
if (remote == "") or (remote == "localhost") or(remote == "127.0.0.1") then 
	stat,summary_result = script_exec2("FASTDB/SUMMARY_JSON",sum_params)
else
	stat,summary_result = script_rexec2(remote,"FASTDB/SUMMARY_JSON",sum_params)
end
if not stat then
	script_result("Status","*ERROR* script FASTDB/SUMMARY_JSON failed.")
	return
end
local sum_tbl = g_json.decode(summary_result["SeriesList"])

---------------------------------
-- 最低値と最高値を取得
---------------------------------
local tmp_arr = {}
for i,v in ipairs(sum_tbl[1]) do -- math.max(),math.min()計算用に集計結果から無効レコードを取り除いたテーブルを一時的に作成
	if v[2] then
		table.insert(tmp_arr,v[2])
	end
end
local max_t = #sum_tbl[1]	-- 集計値のレコード数
local val_min				-- 集計内の最低値
local val_max				-- 集計内の最高値
if #tmp_arr > 0 then
	val_min = math.min(table.unpack(tmp_arr))
	val_max = math.max(table.unpack(tmp_arr))
	log_msg(chart_head .. " 集計レコード数(有効数):" .. #sum_tbl[1] .. "(" .. #tmp_arr ..") 最低値:" .. val_min .. " 最高値:" .. val_max)
	script_result("SumCount",tostring(#sum_tbl[1]))
	script_result("SumMin",tostring(val_min))
	script_result("SumMax",tostring(val_max))
else
	script_result("Status","*ERROR* summary record is empty.")
	graphic2_clear(lcd_ch)
	graphic2_print(lcd_ch,nil,nil,"-- NO DATA --",c_title,title_font,font_size,0,0)
	lcd_disp(0)
	return
end

-----------------------------------------------------------------
-- デバイスの表示エリアサイズに応じたグラフの座標パラメータ設定
-----------------------------------------------------------------
local y_pixels				-- グラフY軸方向に割り当てるピクセル数
local range_min			-- グラフ描画時のスケール最小値
local range_max			-- グラフ描画時のスケール最大値
local gx0					-- グラフ原点位置LCD X座標
local gy0					-- グラフ原点位置LCD Y座標(>= y_pixels にすること)
local tick_size			-- Y軸ラベル&tick間隔を示すピクセル数

if lcd_height >= 240 then		-- ST7789 Color TFT (240x240)等
	y_pixels=180
	range_min = 0
	range_max = y_pixels
	tick_size = 18
	gx0=40
	gy0=200
elseif lcd_height >= 64 then	-- OLED1306(128x64)等
	y_pixels=50
	range_min = 0
	range_max = y_pixels
	tick_size = 10
	gx0=22
	gy0=50
else							-- AQM1248A(128x48)等
	y_pixels=40
	range_min = 0
	range_max = y_pixels
	tick_size = 10
	gx0=22
	gy0=40
end

---------------------------------------------------
-- Y軸用の自動スケールパラメータ計算とラベル作成
---------------------------------------------------

-- 最低値(vmin)と最高値(vmax)から値の範囲を返す。round には値の最低刻み幅を指定する。
function val_range(vmin,vmax,round)
	if (vmin == vmax) and (vmax == 0) then -- 範囲 0 ,値 0 の場合には 0 周辺の値を返す
		vmin = -1
		vmax = 1
	end 
	round = round or 10
	--return round*math.floor(vmin/round),round*math.ceil(vmax/round)
	return round*math.floor(vmin/round)-round,round*math.ceil(vmax/round)+round -- すこし range を広げてチャートを見易くしている
end

-- 値の範囲を指定して tick_count ピクセル分を tick_size ピクセル毎の位置に該当するラベル文字列配列を返す。
function get_ylbl(vmin,vmax,tick_count)
	local arr = {}
	tick_count = tick_count or y_pixels
	for i = 0,tick_count,tick_size do -- グラフY軸方向に割り当てたピクセル数(y_pixels)/tick_size が Y軸ラベル数になる。
		local step = string.format("%g",vmin+(i*(vmax - vmin))/tick_count)
		table.insert(arr,step)
	end
	return arr
end

-- Y軸ラベルの数字をどれぐらいの精度で丸めるかを決める
local round
if g_params["round"] then
	round = tonumber(g_params["round"])
else
	if (math.abs(val_min) < 100) and (math.abs(val_max) < 100) then
		round = 2 -- 温度等の２桁未満のデータ
	else
		round = 10 -- 気圧等の２桁よりも大きなデータ
	end
end
script_result("Round",tostring(round))

-- 集計値を元に計算で求めた自動スケール間隔でY軸ラベルを作成する
local auto_ylbl = {} -- Y軸ラベルに使用する文字列配列。チャート描画時にこのラベル文字列を使用する
range_min,range_max = val_range(val_min,val_max,round)
if g_params["y_min"] then range_min = tonumber(g_params["y_min"]) end
if g_params["y_max"] then range_max = tonumber(g_params["y_max"]) end
script_result("YMin",tostring(range_min))
script_result("YMax",tostring(range_max))
local auto_ylbl = get_ylbl(range_min,range_max)
script_result("YLabel",table.concat(auto_ylbl,",")) 

-- ラベル文字長節約のため 24->12時間表記に変更
function hour12(h)
	if h > 12 then
		return h - 12
	else
		return h
	end
end

local x_grid = {} -- X軸グリッド位置配列(gx0 からの相対値)
local y_grid = {} -- Y軸グリッド位置配列(gy0 からの相対値)

-- 集計範囲に関わらず共通で使用するY軸ラベル,Y軸Tick描画ルーチン
function draw_y_axis_tick()
	-- Y軸
	graphic2_draw_line(lcd_ch,gx0,gy0-y_pixels,gx0,gy0,c_axis,1) 		-- X原点Y軸
	graphic2_draw_line(lcd_ch,gx0+x_mag*max_t,gy0-y_pixels,gx0+x_mag*max_t,gy0,c_axis,1)	-- グラフ右端Y軸

	-- Y軸 tick
	local ytick_w = 2				-- tickの長さ
	local ytick_cnt = #auto_ylbl-1	-- tickの数
	local ytick_pos = 0			-- tick描画位置Y座標
	for i = 1,ytick_cnt do
		ytick_pos = ytick_pos+math.floor(y_pixels/ytick_cnt)
		graphic2_draw_line(lcd_ch,gx0,gy0-ytick_pos,gx0+x_mag*ytick_w,gy0-ytick_pos,c_axis,1)	-- X原点のY軸 tick
		graphic2_draw_line(lcd_ch,gx0+x_mag*(max_t-ytick_w),gy0-ytick_pos,gx0+x_mag*max_t,gy0-ytick_pos,c_axis,1)	-- グラフ右端のY軸 tick
		table.insert(y_grid,ytick_pos)
	end

	-- Y軸ラベルを描画する座標と文字列を格納する配列を一時的に作成
	local ylbl = {} -- {<ラベル位置X軸オフセット>,<ラベル位置Y軸オフセット>,<ラベル文字列>} テーブルの配列
	if #auto_ylbl == 11 then
		for i = 0,10 do
			local item = {-40,-7-(i*18),""}
			table.insert(ylbl,item)
		end
	elseif #auto_ylbl == 6 then
		for i = 0,5 do
			local item = {-22,-6-(i*9),""}
			table.insert(ylbl,item)
		end
	else
		for i = 0,4 do
			local item = {-22,-6-(i*9),""}
			table.insert(ylbl,item)
		end
	end
	-- 自動で計算して作成した Y軸ラベルを一時的な配置用配列に設定
	for i,v in ipairs(auto_ylbl) do
		ylbl[i][3] = v
	end
	-- Y軸ラベル描画
	for i,v in ipairs(ylbl) do
		graphic2_print(lcd_ch,gx0+v[1],gy0+v[2],v[3],c_label,tick_font,font_size,0,1)
	end
end

----------------------------------------------
-- X/Y軸,ラベル,tick,タイトル描画(背景ページ)
----------------------------------------------
if range == "week" then
	-- グラフタイトル
	if chart_head ~= "" then
		local title_x	-- タイトル位置
		if lcd_height >= 240 then
			title_x = gx0+50
			graphic2_print(lcd_ch,title_x,2,chart_head .. ' ' .. chart_head2,c_title,title_font,font_size,0,1)
		else
			title_x = gx0+max_t+3
			graphic2_print(lcd_ch,title_x,2,chart_head,c_title,mini_font,font_size,0,1)
			graphic2_print(lcd_ch,title_x,15,chart_head2,c_title,mini_font,font_size,0,1)
		end
	end

	-- X軸
	graphic2_draw_line(lcd_ch,gx0,gy0,gx0+x_mag*max_t,gy0,c_axis,1)

	-- X軸ラベル作成
	local weekstr = {"日","月","火","水","木","金","土","日","月","火","水","木","金","土"}
	local year,month,day = stat_check(str_to_datetime(sum_tbl[1][1][1]))
	local first_wday = stat_check(day_of_week(year,month,day))
	local xlbl = {}
	for i = first_wday,first_wday + 6,1 do
		table.insert(xlbl,weekstr[i])
	end
	script_result("XLabel",table.concat(xlbl,",")) 

	-- X軸ラベル
	local xlbl_spw = 12	-- ラベル間隔
	local xlbl_offset = 2	-- ラベル開始位置X軸
	local xlbl_y_offset = 0
	for i,v in ipairs(xlbl) do
		if (lcd_height-gy0) > 8 then
			xlbl_y_offset = 2
		end
		graphic2_print(lcd_ch,gx0+x_mag*(xlbl_offset+xlbl_spw*(i-1)),gy0+xlbl_y_offset,v,c_label,mini_font,font_size,0,1)
	end

	-- X軸 tick
	local xtick_h = 2		-- tickの長さ
	local xtick_spw = 3	-- tick間隔
	for i = 1,28 do
		if math.fmod(i,4) == 0 then -- Xラベル部分の tick は 2倍に伸ばす
			graphic2_draw_line(lcd_ch,gx0+x_mag*xtick_spw*i,gy0,gx0+x_mag*xtick_spw*i,gy0-x_mag*2*xtick_h,c_axis,1)
			table.insert(x_grid,x_mag*xtick_spw*i)
		else
			graphic2_draw_line(lcd_ch,gx0+x_mag*xtick_spw*i,gy0,gx0+x_mag*xtick_spw*i,gy0-x_mag*xtick_h,c_axis,1)
		end
	end

	draw_y_axis_tick()
end

if range == "6h" then
	-- グラフタイトル
	if chart_head ~= "" then
		local title_x = gx0+x_mag*(max_t+2)		-- タイトル位置
		graphic2_print(lcd_ch,title_x,0,chart_head,c_title,title_font,font_size,0,1)
		graphic2_print(lcd_ch,title_x,24,chart_head2,c_title,title_font,font_size,0,1)
	end

	-- X軸
	graphic2_draw_line(lcd_ch,gx0,gy0,gx0+x_mag*max_t,gy0,c_axis,1)

	-- X軸ラベル作成
	local dummy1, dummy2, dummy3, first_hour, dummy4, dummy5 = stat_check(str_to_datetime(sum_tbl[1][1][1]))
	local xlbl = {} -- ラベル文字列
	local first_hour_tick_cnt = 0 -- 集計データ先頭のタイムスタンプと同じ時間(Hour)を示すレコードが、先頭から何レコード連続するかを示す
	local current_hour = first_hour
	table.insert(xlbl,tostring(hour12(current_hour)))
	-- 時間を表すラベル文字列を検索結果のタイムスタンプから抽出する(1Hr単位で6または7つになる)
	for i = 1,max_t do
		local check_hour
		dummy1, dummy2, dummy3, check_hour, dummy4, dummy5 = stat_check(str_to_datetime(sum_tbl[1][i][1]))
		if check_hour ~= current_hour then
			current_hour = check_hour
			table.insert(xlbl,tostring(hour12(current_hour)))
		end
		if check_hour == first_hour then
			first_hour_tick_cnt = first_hour_tick_cnt + 1
		end
	end
	script_result("XLabel",table.concat(xlbl,",")) 

	-- X軸ラベル
	local xlbl_spw = 12	-- ラベル間隔
	local xlbl_offset = -2		-- ラベル開始位置X軸オフセット
	local xlbl_pos = xlbl_offset	-- ラベル位置
	-- X軸の一番目のラベルは時間途中から始まる場合があるので最初の時間(Hour)が何Tick続くかを調べる
	-- 12tick 続いた場合には最初の時間からラベルを作成して、それ以外の場合は次の時間からのラベルのみを表示
	for i,v in ipairs(xlbl) do
		if (i ~= 1) or ((i == 1) and (first_hour_tick_cnt == 12)) then
			graphic2_print(lcd_ch,gx0+x_mag*xlbl_pos,gy0,v,c_label,tick_font,font_size,0,1)
		end
		if i == 1 then
			xlbl_pos = xlbl_pos + first_hour_tick_cnt
		else
			xlbl_pos = xlbl_pos + xlbl_spw
		end
	end
	
	-- X軸 tick
	local xtick_h = 2		-- tickの長さ
	local xtick_cnt = 0
	for i = first_hour_tick_cnt,max_t do -- １番目のラベル時刻を含む右側の Tick を描画
		if math.fmod(xtick_cnt,12) == 0 then -- 時間の区切りはすこし長めの Tick にする
			graphic2_draw_line(lcd_ch,gx0+x_mag*i,gy0,gx0+x_mag*i,gy0-xtick_h*2,c_axis,1)
			table.insert(x_grid,x_mag*i)
		else
			if math.fmod(xtick_cnt,3) == 0 then 
				graphic2_draw_line(lcd_ch,gx0+x_mag*i,gy0,gx0+x_mag*i,gy0-xtick_h,c_axis,1)
			end
		end
		xtick_cnt = xtick_cnt + 1
	end
	xtick_cnt = 0							
	for i = first_hour_tick_cnt,0,-1 do -- １番目のラベル時刻よりも左側の Tick を描画
		if math.fmod(xtick_cnt,3) == 0 then
			graphic2_draw_line(lcd_ch,gx0+x_mag*i,gy0,gx0+x_mag*i,gy0-xtick_h,c_axis,1)
		end
		xtick_cnt = xtick_cnt + 1
	end

	draw_y_axis_tick()
end

if (range == "day") or (range == "2d") then
	-- グラフタイトル
	if chart_head ~= "" then
		local title_x = x_mag*40		-- タイトル位置
		graphic2_print(lcd_ch,title_x,0,chart_head .. " " .. chart_head2,c_title,title_font,font_size,0,1)
	end

	-- X軸
	graphic2_draw_line(lcd_ch,gx0,gy0,gx0+x_mag*max_t,gy0,c_axis,1)
	
	-- X軸ラベル
	local xlbl_spw = 16			-- ラベル間隔
	local xlbl_offset = -3			-- ラベル開始位置X軸
	local xlbl 					-- ラベル文字列
	if range == "day" then
		xlbl = {"0","4","8","12","16","20","0"}
	end
	if range == "2d" then
		xlbl = {"0","8","16","0","8","16","0"}
	end
	for i,v in ipairs(xlbl) do
		graphic2_print(lcd_ch,gx0+x_mag*(xlbl_offset+xlbl_spw*(i-1)),gy0,v,c_label,tick_font,font_size,0,1)
	end
	script_result("XLabel",table.concat(xlbl,",")) 
	
	-- X軸 tick
	local xtick_h = 2		-- tickの長さ
	local xtick_spw = 4	-- tick間隔
	for i = 1,24 do
		if math.fmod(i,4) == 0 then -- Xラベル部分の tick は 2倍に伸ばす
			graphic2_draw_line(lcd_ch,gx0+x_mag*xtick_spw*i,gy0,gx0+x_mag*xtick_spw*i,gy0-2*xtick_h,c_axis,1)
			table.insert(x_grid,x_mag*xtick_spw*i)
		else
			graphic2_draw_line(lcd_ch,gx0+x_mag*xtick_spw*i,gy0,gx0+x_mag*xtick_spw*i,gy0-xtick_h,c_axis,1)
		end
	end
	
	draw_y_axis_tick()
end

------------------
-- グリッド描画
------------------
if (#x_grid >= 1) and (#y_grid >= 1) and (grid ~= "0") then
	for i,gr_x in ipairs(x_grid) do
		for j,gr_y in ipairs(y_grid) do
			if (gr_x ~= 0) and (gr_y ~= 0) then
				if grid == "1" then
					graphic2_draw_pixel(lcd_ch,gx0+gr_x,gy0-gr_y,c_grid,1)
				elseif grid == "2" then
					graphic2_draw_line(lcd_ch,gx0+gr_x-x_mag,gy0-gr_y,gx0+gr_x+x_mag,gy0-gr_y,c_grid,1)
					graphic2_draw_line(lcd_ch,gx0+gr_x,gy0-gr_y-x_mag,gx0+gr_x,gy0-gr_y+x_mag,c_grid,1)
				end
			end
		end
	end
end

lcd_disp(0,1) -- 背景ページ(1)に描画した内容をフォアグランドページ(0)にコピーする

-------------------------------------------------------------------------------
-- チャート中の(X軸:t0,Y軸:v0)から (X軸:t1,Y軸:v1) に直線を描画する
-- v0 または v1 のどちらが nil の場合は点を描画する
-- t0,t1: 1..max_t
-- v0,v1: 0..y_pixels or nil (float 値も可)
-------------------------------------------------------------------------------
function draw_chart_line(t0,t1,v0,v1)
	if (not t0) or (not t1) then return end
	local cx0 = gx0 + math.floor(x_mag*max_t*(t0-1)/(max_t-1))
	local cx1 = gx0 + math.floor(x_mag*max_t*(t1-1)/(max_t-1))
	local cy0,cy1
	if v0 then cy0 = gy0 - math.floor(y_pixels*v0/y_pixels) end
	if v1 then cy1 = gy0 - math.floor(y_pixels*v1/y_pixels) end
	if v0 and (not v1) then
		graphic2_draw_pixel(lcd_ch,cx0,cy0,c_line,0)
	elseif (not v0) and v1 then
		graphic2_draw_pixel(lcd_ch,cx1,cy1,c_line,0)
	elseif v0 and v1 then
		--log_msg("line(" .. cx0 .. "," .. cy0 ..")-("  .. cx1 .. "," .. cy1 .. ")")
		graphic2_draw_line(lcd_ch,cx0,cy0,cx1,cy1,c_line,0)
	end
end

-- 値を元の範囲から別の範囲にマッピングして変換する
function vmap(x,in_min,in_max,out_min,out_max)
	if not x then return x end
	return math.floor((x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min)
end

---------------------------------
-- チャート描画
---------------------------------
local val_str
local prev_v
for i,v in ipairs(sum_tbl[1]) do
	if v[2] then
		val_str = tostring(v[2])
	else
		val_str = ""
	end
	--log_msg("summary[" .. i .. "] datetime = " .. v[1] .. " val = " .. val_str)
	draw_chart_line(i-1,i,vmap(prev_v,range_min,range_max,0,y_pixels),vmap(v[2],range_min,range_max,0,y_pixels))
	prev_v = v[2]
end

lcd_disp(0)

script_result("Status","OK")
