
--[[

●機能概要

Scroll pHAT HD LEDマトリックス表示器に文字列リストを順に表示する。
表示する文字列データはグローバル共有文字列リスト Channelに設定されているものを使用する。

文字列リストを取り出して、各文字列を順にLEDマトリックスに表示する。

長い文字列はスクロール表示される。表示可能な文字は ASCII と日本語で、それ以外の
文字は表示できない。


●参照するグローバル共有文字列リスト Channel

---------------------------------------------------------------------------------
キー値                      値              
---------------------------------------------------------------------------------
SCROLLPHATHD_STRINGS	文字列リスト

表示するメッセージ文字列リストを設定する。
設定可能なリスト中の各文字列の最大長は英数字の場合に約 256文字です。(Max 1536ピクセル)

●参照するグローバル共有データ

---------------------------------------------------------------------------------
キー値                      値              
---------------------------------------------------------------------------------
SCROLLPHATHD_FORCE_UPDATE	"1" 
							セットするデータ値は何でも構わない

この共有変数に値をセットすると、現在のスクロール表示を中断して最新の文字列リストを再ロードする。
この機能を使用しない場合には、スクロール表示中の文字列リストの最後の文字列が表示された後、
自動的に最新の文字列リストがロードされます。

●セットアップ

このスクリプトは ScrollpHAT HD (17x7 pixel) PIMORONI LEDマトリックス出力用ライブラリ関数を
使用します。インストール直後はライブラリが無効になっていますので、ライブラリの
ディレクトリ名をリネームして有効にしてください。方法は、ライブラリを格納しているディレクトリ名
"preload/100_RASPI/_scrollphathd" の、先頭についたアンダースコアを取り除いて
"preload/100_RASPI/scrollphathd" にリネームした後 abs_agent を再起動させてください。

●備考

このスクリプトは無限ループに入って終了しないので、必ず別スレッドで起動してください。

●変更履歴

2019/12/4	   日本語フォント検索時にグローバル共有変数用 API を利用するようにした

2018/05/17     日本語フォント表示機能を追加

2018/05/06     bright パラメータを追加

2018/04/14     初版作成

copyright(c) 2018 All Blue System

]]

---------------------------------
-- ２重起動防止用チェック
---------------------------------
if not exclusive_check(g_script) then
    log_msg("*ERROR* exclusive_check() failed. script = " .. g_script,file_id)
    return
end
log_msg("start..   TaskID = " .. g_taskid,g_script)

-----------------------------------------------------------------------------------------------------------------
-- 指定したグローバル共有変数に何か値がセットされるか、もしくは指定されたカウント値 x 10ms 経過するまで
-- 内部でウェイトする。セットされたグローバル共有変数の値を返す。タイムアウト発生時は "" 空文字列が返る。
-----------------------------------------------------------------------------------------------------------------
function global_set_wait(global_name,max_wait_cntr)
	local stat,new_val
	local cntr = 0
	repeat
		wait_time(10)
		cntr = cntr + 1
		stat,new_val = get_shared_data(global_name,true) -- 共有変数の取得を試みた後、常に対象の共有変数を削除する
	until (cntr >= max_wait_cntr) or (new_val ~= "")
	return new_val
end

---------------------------------
-- デバイスとタスク内変数初期化
---------------------------------
if not raspi_i2c_clock(1,1500) then error() end	-- I2C bus1 クロック設定 100KHz = 150MHz / 1500
scrollphathd_try_init()								-- ScrollpHATHD 初期化
local msglist_var = "SCROLLPHATHD_STRINGS"			-- 表示する文字列リストを取得するグローバル共有文字列リスト Channel
local forceupd_var = "SCROLLPHATHD_FORCE_UPDATE"   -- スクロール表示を強制的に中断して文字列データを再読み込みさせるフラグ

-----------------------------------------------------------------------------------------------------------------
-- メインループ。無限ループを停止させる場合には script_kill() または "agent_task -k <taskid>" コマンドを使用する
-----------------------------------------------------------------------------------------------------------------
local pixel_data			-- 描画時のピクセルデータを一時的に保持する
local pixel_idx			-- 描画時のピクセルデータを一時的に保持する
local org_x = 0			-- 文字列の描画開始座標
local org_y = 0			-- 文字列の描画開始座標
local sw = 0				-- 文字列のピクセル幅
local sh = 0				-- 文字列のピクセル高
local line_wait = 100		-- 文字列リスト項目毎にウェイトする時間 (x10 ms)
local first_wait = 50		-- スクロール表示時の最初にウェイトする時間 (x10 ms)
local inter_wait = 3		-- スクロール中のウェイトする時間 (x10 ms)
local max_x = 0
local force_update = ""	-- SCROLLPHATHD_FORCE_UPDATE グローバル共有変数の値
local stat,msglist
local bright = 0.5			-- 描画時の明るさ。0 から 1.0 までの範囲で指定
--local font = 2 									-- 5x7smooth フォント
local font_name = "MISAKI_FONTX"	-- グローバル共有メモリに保存されている日本語フォントデータ
while true do
	stat, msglist = get_shared_strlist(msglist_var)
	if (not stat) or (#msglist == 0) then
		msglist = {""}
	end
	for k,disp_msg in ipairs(msglist) do
		pixel_data = {}
		pixel_idx = {}
		scrollphathd_clear()
		-- sw,sh = scrollphathd_draw_string(pixel_idx,pixel_data,org_x,org_y,disp_msg,font,bright) -- 英数字のみ
		sw,sh = scrollphathd_draw_jstring(pixel_idx,pixel_data,org_x,org_y,disp_msg,font_name,bright) -- 英数字と日本語
		scrollphathd_draw_commit(pixel_idx,pixel_data)
		scrollphathd_transfer(0,0)
		scrollphathd_display(0)
		max_x = org_x + sw
		if (max_x > 17) then -- スクロール表示が必要か?
			scrollphathd_transfer(0,0) -- 最初の１画面分を表示
			force_update = global_set_wait(forceupd_var,first_wait)
			if force_update ~= "" then goto SCROLL_FINISH end
			if max_x > 1535 then max_x = 1535 end
			for x0 = 1,max_x - 17 do -- 画面に表示する部分を１ピクセル右にずらす
				scrollphathd_transfer(0,x0)
				force_update = global_set_wait(forceupd_var,inter_wait)
				if force_update ~= "" then goto SCROLL_FINISH end
			end
		end
		force_update = global_set_wait(forceupd_var,line_wait)
		if force_update ~= "" then goto SCROLL_FINISH end
	end

::SCROLL_FINISH::

end

