
--[[

●機能概要

Scroll pHAT HD LEDマトリックス表示器に文字列を表示する。
表示する文字列データはグローバル共有変数に設定されているものを使用する。

長い文字列はスクロール表示される。表示可能な文字は ASCIIと日本語で、それ以外の
文字は表示できない。


●参照するグローバル共有変数

---------------------------------------------------------------------------------
キー値                      値              
---------------------------------------------------------------------------------
SCROLLPHATHD_MESSAGE	"<message string>"

表示するメッセージ文字列を設定する。
設定可能な文字列の最大長は英数字の場合に約 256文字です。(Max 1536ピクセル)

●セットアップ

このスクリプトは ScrollpHAT HD (17x7 pixel) PIMORONI LEDマトリックス出力用ライブラリ関数を
使用します。インストール直後はライブラリが無効になっていますので、ライブラリの
ディレクトリ名をリネームして有効にしてください。方法は、ライブラリを格納しているディレクトリ名
"preload/100_RASPI/_scrollphathd" の、先頭についたアンダースコアを取り除いて
"preload/100_RASPI/scrollphathd" にリネームした後 abs_agent を再起動させてください。

●備考

このスクリプトは無限ループに入って終了しないので、必ず別スレッドで起動してください。

●変更履歴

2019/12/4	   日本語フォント検索時にグローバル共有変数用 API を利用するようにした

2018/05/17     日本語フォント表示機能を追加

2018/05/06     bright パラメータを追加

2018/04/06     初版作成

copyright(c) 2018 All Blue System

]]

---------------------------------
-- ２重起動防止用チェック
---------------------------------
if not exclusive_check(g_script) then
    log_msg("*ERROR* exclusive_check() failed. script = " .. g_script,g_script)
    return
end
log_msg("start..   TaskID = " .. g_taskid,g_script)

-----------------------------------------------------------------------------------------------------------------
-- 指定したグローバル共有変数の値が変更されるか、もしくは指定されたカウント値 x 10ms 経過するまで内部でウェイト
-- 最後に取得したグローバル共有変数の値を返す。
-----------------------------------------------------------------------------------------------------------------
function global_change_wait(global_name,start_val,max_wait_cntr)
	local stat,new_val
	local cntr = 0
	repeat
		wait_time(10)
		cntr = cntr + 1
		stat,new_val = get_shared_data(global_name)
	until (cntr >= max_wait_cntr) or (start_val ~= new_val)
	return new_val
end

---------------------------------
-- デバイスとタスク内変数初期化
---------------------------------
if not raspi_i2c_clock(1,1500) then error() end	-- I2C bus1 クロック設定 100KHz = 150MHz / 1500
scrollphathd_try_init()								-- ScrollpHATHD 初期化
local message_var = "SCROLLPHATHD_MESSAGE"			-- 表示する文字列データを取得するグローバル共有変数

-----------------------------------------------------------------------------------------------------------------
-- メインループ。無限ループを停止させる場合には script_kill() または "agent_task -k <taskid>" コマンドを使用する
-----------------------------------------------------------------------------------------------------------------
local pixel_data			-- 描画時のピクセルデータを一時的に保持する
local pixel_idx			-- 描画時のピクセルデータを一時的に保持する
local org_x = 0			-- 文字列の描画開始座標
local org_y = 0			-- 文字列の描画開始座標
local disp_msg = "???"		-- 現在表示中の文字列データ
local new_msg = ""			-- 最後に取得した最新の文字列データ
local sw = 0				-- 文字列のピクセル幅
local sh = 0				-- 文字列のピクセル高
local first_wait = 50		-- スクロール表示時の最初と最後にウェイトする時間 x10ms
local inter_wait = 3		-- スクロール中のウェイトする時間 x10ms
local max_x = 0
local bright = 0.5			-- 描画時の明るさ。0 から 1.0 までの範囲で指定
--local font = 2 									-- 5x7smooth フォント
local font_name = "MISAKI_FONTX"	-- グローバル共有メモリに保存されている日本語フォントデータ
while true do
	if disp_msg ~= new_msg then -- 新しい文字列データがセットされたので描画する
		disp_msg = new_msg
		pixel_data = {}
		pixel_idx = {}
		scrollphathd_clear()
		-- sw,sh = scrollphathd_draw_string(pixel_idx,pixel_data,org_x,org_y,disp_msg,font,bright) -- 英数字のみ
		sw,sh = scrollphathd_draw_jstring(pixel_idx,pixel_data,org_x,org_y,disp_msg,font_name,bright) -- 英数字と日本語
		scrollphathd_draw_commit(pixel_idx,pixel_data)
		scrollphathd_transfer(0,0)
		scrollphathd_display(0)
	end
	max_x = org_x + sw
	if (max_x > 17) then -- スクロール表示が必要か?
		scrollphathd_transfer(0,0) -- 最初の１画面分を表示
		new_msg = global_change_wait(message_var,disp_msg,first_wait)
		if disp_msg ~= new_msg then goto SCROLL_FINISH end
		if max_x > 1535 then max_x = 1535 end
		for x0 = 1,max_x - 17 do -- 画面に表示する部分を１ピクセル右にずらす
			scrollphathd_transfer(0,x0)
			new_msg = global_change_wait(message_var,disp_msg,inter_wait)
			if disp_msg ~= new_msg then goto SCROLL_FINISH end
		end
	end
	new_msg = global_change_wait(message_var,disp_msg,first_wait)
	
::SCROLL_FINISH::

end

