
file_id = "FIRMATA_LIB"

--[[
***************************************************************************************************

	<機能概要>
	
	与えられたFIRMATA フレームデータを解析して文字列データと配列データに変換する

	<関数定義>

	stat [,msg [,data] ] = parse_firmata(frame)

	<パラメータとリターン値>

	stat: Boolean			成功した場合には true,失敗した場合には false が返る
	
	msg: Table of String	フレームデータ中のメッセージが文字列で入る
							フレームパケット毎に格納されるキー名と内容が予め決められている
							詳細は、ソース中のコメントを参照の事

	data: Table of Number	フレームデータ中に数値配列が格納されていた場合に、そのデータ配列が入る
							フレームパケット毎に、格納される内容が予め決められている
							詳細は、ソース中のコメントを参照の事

	frame: String			FIRMATA バイナリデータを 16進数文字列にしたもの

	<備考>

	この関数は、予め定義された代表的なパケット種別のみがインプリメントされています。

	必要なパケット種別がサポートされていない場合には、このソースを修正または追加して
	機能を拡張してください。

    この関数は DeviceServer システム内部からはコールされませんので
	自由に改変することができます。(デフォルトの SERIAL_FIRMATA イベントハンドラを除く)

***************************************************************************************************
]]

function parse_firmata(frame)
	local ret = {}
	local rec = hex_to_tbl(frame)
	local stat,str

	--[[
	---------------------------------------------------------------------------------
	data packet		| 0xE0 - 0xEF   |
					| LSB(bits 0-6) |
					| MSB(bits 7-13)|
	---------------------------------------------------------------------------------

	msg[]
	---------------------------------------------------------------------------------
	キー値			値		            							値の例
	---------------------------------------------------------------------------------
	Command			"ANALOG_MESSAGE"								"ANALOG_MESSAGE"
	PinNumber		pin#											"0"
	PinValue		value(10進数)											"1023"
	---------------------------------------------------------------------------------

	]]	

	if bit_and(rec[1],0xf0) == 0xe0 then 
		ret["Command"] = "ANALOG_MESSAGE"
		ret["PinNumber"] = tostring(bit_and(rec[1],0x0f))
		local tmp_arr = hex72_to_tbl(string.sub(frame,3,-1),2)
		ret["PinValue"] = tostring(tmp_arr[1])
		return true,ret
	end


	--[[
	---------------------------------------------------------------------------------
	data packet		| 0x90 - 0x9F   |
					| LSB(bits 0-6) |
					| MSB(bits 7-13)|
	---------------------------------------------------------------------------------

	msg[]
	---------------------------------------------------------------------------------
	キー値			値		            							値の例
	---------------------------------------------------------------------------------
	Command			"DIGITAL_MESSAGE"										"DIGITAL_MESSAGE"
	PortNumber		pin#													"0"
	PortValue		value(16進数)											"0A"
	---------------------------------------------------------------------------------

	]]	

	if bit_and(rec[1],0xf0) == 0x90 then 
		ret["Command"] = "DIGITAL_MESSAGE"
		ret["PortNumber"] = tostring(bit_and(rec[1],0x0f))
		local tmp_arr = hex72_to_tbl(string.sub(frame,3,-1),2)
		ret["PortValue"] = bit_tohex(tmp_arr[1],2)
		return true,ret
	end



	--[[
	---------------------------------------------------------------------------------
	data packet		| 0xF0            | (START_SYSEX)
					| 0x71            | 
					| string[0]	      | (7bits char)
					| string[1]	      | (7bits char)
					| ..              |
					| ..              |
					| string[n]	      | (7bits char)
					| 0xF7            | (END_SYSEX)

	msg[]
	---------------------------------------------------------------------------------
	キー値			値		            							値の例
	---------------------------------------------------------------------------------
	Command			"SYSEX_STRING"									"SYSEX_STRING"
	String			user string										"hello world!!"
	---------------------------------------------------------------------------------
	]]	

	if (rec[1] == 0xf0) and (rec[2] == 0x71) then 
		ret["Command"] = "SYSEX_STRING"
		stat,str = firmata_str_decode(string.sub(frame,5,-3))
		if not stat then return false end
		ret["String"] = str
		return true,ret
	end

	--[[
	---------------------------------------------------------------------------------
	data packet		| 0xF0            | (START_SYSEX)
					| 0x79            | 
					| major version#  |
					| minor version#  |
					| firmware_name[0]| (7bits char)
					| firmware_name[1]| (7bits char)
					| ..              |
					| ..              |
					| firmware_name[n]| (7bits char)
					| 0xF7            | (END_SYSEX)

	msg[]
	---------------------------------------------------------------------------------
	キー値			値		         								値の例
	---------------------------------------------------------------------------------
	Command			"REPORT_FIRMWARE"								"REPORT_FIRMWARE"
	MajorVersion	major version#									"2"
	MinorVersion	minor version#									"3"
	FirmwareName	firmware name									"std_firmata"
	---------------------------------------------------------------------------------
	]]	

	if (rec[1] == 0xf0) and (rec[2] == 0x79) then 
		ret["Command"] = "REPORT_FIRMWARE"
		ret["MajorVersion"] = tostring(rec[3])
		ret["MinorVersion"] = tostring(rec[4])
		stat,str = firmata_str_decode(string.sub(frame,9,-3))
		if not stat then return false end
		ret["FirmwareName"] = str
		return true,ret
	end

	--[[
	---------------------------------------------------------------------------------
	data packet		| 0xF0            | (START_SYSEX)
					| 0x77            | 
					| slave addr LSB  |
					| slave addr MSB  |
					| register LSB    |
					| register MSB    |
					| i2c_data[0] LSB |
					| i2c_data[0] MSB |
					| ..              |
					| ..              |
					| 0xF7            | (END_SYSEX)


	msg[]
	---------------------------------------------------------------------------------
	キー値			値	           									値の例
	---------------------------------------------------------------------------------
	Command			"I2C_REPLY"										"I2C_REPLY"
	SlaveAddress	slave address									"50"
	Register		register										"00"
	---------------------------------------------------------------------------------

	data[]			フレーム中の i2c_data[] 列を数値配列に変換したものが入る

	]]

	if (rec[1] == 0xf0) and (rec[2] == 0x77) then 
		ret["Command"] = "I2C_REPLY"
		local tmp_arr = hex72_to_tbl(string.sub(frame,5,-3),2)
		ret["SlaveAddress"] = bit_tohex(tmp_arr[1],2)
		ret["Register"] = bit_tohex(tmp_arr[2],2)
		data = {}
		for i = 3,#tmp_arr,1 do
			table.insert(data,tmp_arr[i]) 
		end
		return true,ret,data
	end


	------------------------------------------------
	-- ここに追加のパケット処理を記述してください --
	------------------------------------------------







	--------------------------
	-- パケット処理に失敗した
	--------------------------
	return false
end

