
--[[

●概要

MQTT デバイスの 高速ダイレクトコマンド(dx_command)を有効にしたときに利用できるライブラリ関数

Ver 1.3

ChangeLog:

2021/4/22	dxcmd_param_add_long() 追加

2020/9/7  	dxcmd_exec_reply_get() 追加

2020/8/24	dxcmd_exec_stat_check(),dxcmd_param_add_byte(),dxcmd_param_add_word(),dxcmd_param_append()追加

copyright(c) 2020  All Blue System

]]


--[[

●機能概要

バイナリコマンドを MGCP デバイスに UDP データグラムソケット経由で送信する。

●関数定義

stat, rpl_arr = mgcp_dx_command(node, req_arr [,noreply [,timeout] ] )


●リクエストパラメータ
---------------------------------------------------------------------------------
パラメータ名	説明	            									値の例
---------------------------------------------------------------------------------
node:string		リクエストコマンドの送信対象のノード名				"ESP32Node1"

req_arr:Table [1..#max_item] of Number
				リクエストフレーム(コマンドとパラメータ部分)の数値データ配列
																	{0xFE,0x01}
																	
noreply:boolean	リプライメッセージを受信しないことを指定する。true または false を指定。
				パラメータ省略時は false でリプライ受信まで内部で wait する。
																	false

timeout:number	リプライメッセージ受信までのタイムアウト(ms) を指定する。
				パラメータ省略時は 1000ms(1秒) が指定されます。		500

●リターン値
---------------------------------------------------------------------------------
リターン値名	説明	            									値の例
---------------------------------------------------------------------------------
stat:boolean	MGCP リクエストの送信とリプライ受信処理が成功した場合には true が入る。
				パラメータ指定が間違っている場合や、リプライ受信時にタイムアウト
				した場合には false が返ります。

				指定したサービスが提供されていない等で、サーバー側のコマンド処理が失敗した
				場合でも、リプライメッセージを受信した時点で true が返る事に注意してください。

				サーバー側のコマンド処理が正常に終了したかどうかは、reply で返される16進数文字列中の
				の Status バイトの値が 01 になっているかで判断できます。
																		true

rpl_arr:Table [1..#max_item] of Number	
				サーバーから返されたリプライフレーム(ステータスとデータ部分)の数値データ配列
																		{0x01}

●リクエストパケット

offset 値に 1 を足した値が Lua のインデックス値になる点に注意

 -----------------------------------------------------------
 offset	item				(request packet)
 -----------------------------------------------------------
  0		<StartDelimiter>	0x7E
  1		<Length MSB>		<BootCounter> から <Params> までのバイト数
  2		<Length LSB>
  3		<BootCounter>		下位 1byte分のみチェック
  4		<MessageID>     	リプライ無しの場合は 0x00, リプライ要求の場合は 0x00 以外の値を格納
  5		<Command>			コマンド種別
 (6)	<Params>		    コマンドパラメータ#0(必要な場合のみ)
 (7)	<Params>		    コマンドパラメータ#1(必要な場合のみ)
 ..
 ..
 xx		<CheckSum>			<BootCounter> から <Params> までのチェックサム値


●リプライパケット
 -----------------------------------------------------------
 offset	item				(reply packet)
 -----------------------------------------------------------
  0		<StartDelimiter>	0x7E
  1		<Length MSB>		<MessageID> から <Data> までのバイト数
  2		<Length LSB>
  3		<MessageID>      	Request Packet 中に指定された<MessageID> をそのまま格納, 0x00 以外の値
  4		<Status>			実行結果(0: error, 1:success)
 (5)	<Data>		    	リプライデータ#0(必要な場合のみ)
 (6)	<Data>		    	リプライデータ#1(必要な場合のみ)
 ..
 ..
 xx		<CheckSum>			<MessageID> から <Data> までのチェックサム値

]]

function calc_mgcp_frame_checksum(tbl) 
	local checksum = 0 
	for i = 4,#tbl,1 do
		checksum = checksum + tbl[i]
		checksum = bit_and(0xff,checksum)
	end
	return checksum
end

function mgcp_dx_command(...)
	local logmod = "mgcp_dx_command()"
	local timeout = 1000
	local noreply = false
	local clientid = ""
	local node = ""
	local req_arr = {}
	local argv = {...}
	local argc = #argv
	local messge_id
	local max_recv_size = 1024
	local stat
	local rpl_packet
	local rpl_frame = {}

	------------------------------------------------------------------------------------------------
	-- パラメータ設定
	------------------------------------------------------------------------------------------------
	if argc < 2 then
		return false
	end

	if type(argv[1]) == "string" then
		node = argv[1]
	else
		return false
	end

	if type(argv[2]) == "table" then
		req_arr = argv[2]
	else
		return false
	end

	if (argc >= 3) and (type(argv[3]) == "boolean") then
		noreply = argv[3]
	end

	if (argc == 4) and (type(argv[4]) == "number") then
		timeout = argv[4]
	end

	local ip = stat_check(get_shared_data("$IP_" .. node))
	if ip == "" then
		log_msg("*ERROR* unknown node, node = " .. node,logmod)
		return false
	end

	local dx_port = stat_check(get_shared_data("$DX_PORT_" .. node))
	if dx_port == "" then
		log_msg("*ERROR* unknown dx_port, node = " .. node,logmod)
		return false
	end
	local port = tonumber(dx_port)

	local reboot = stat_check(get_shared_data("$REBOOT_" .. node))
	if reboot == "" then
		log_msg("*ERROR* boot_cntr not found, node = " .. node,logmod)
		return false
	end
	local boot_cntr = tonumber(reboot)
	
	if noreply then
		messge_id = 0
	else
		messge_id = math.random(1,255)
	end

	------------------------------------------------------------------------------------------------
	-- リクエストパケット作成
	------------------------------------------------------------------------------------------------
	local req_packet = {}
	local frame_len = #req_arr + 2
	table.insert(req_packet,0x7e)
	table.insert(req_packet,bit_and(bit_lshift(frame_len,8),0xff))
	table.insert(req_packet,bit_and(frame_len,0xff))
	table.insert(req_packet,bit_and(0xff,boot_cntr))
	table.insert(req_packet,messge_id)
	for i,v in ipairs(req_arr) do
		table.insert(req_packet,v)
	end
	local check = calc_mgcp_frame_checksum(req_packet)
	table.insert(req_packet,check)

	------------------------------------------------------------------------------------------------
	-- パケット送信とリプライ受信
	------------------------------------------------------------------------------------------------
	if noreply then
		return udp_send_binary(ip,port,req_packet)
	else
		stat,rpl_packet = udp_send_recv_binary(ip,port,req_packet,max_recv_size,timeout)
		if not stat then 
			log_msg("*ERROR* udp_send_recv_binary() failed",logmod)
			return false
		end
		local check_sum = table.remove(rpl_packet)
		if check_sum ~= calc_mgcp_frame_checksum(rpl_packet) then
			log_msg("*ERROR* checksum error",logmod)
			return false
		end
		frame_len = bit_lshift(rpl_packet[2],8) + rpl_packet[3] - 1
		for i = 1,frame_len,1 do
			table.insert(rpl_frame,rpl_packet[i + 4])
		end
		return true,rpl_frame
	end

end

------------------------------------------------------------------------------------------
-- ダイレクトコマンドのリクエストフレームを送信して結果ステータスをチェックする
-- 正常にリプライを受信した後、リプライデータ中のステータスバイトが 0x01 の時のみ
-- true を返しそれ以外は false を返す
------------------------------------------------------------------------------------------
function dxcmd_exec_stat_check(node,frame)
	local dxstat,reply = mgcp_dx_command(node,frame,false,500)
	if dxstat then
		if reply[1] == 0x01 then
			return true
		else
			return false
		end
	else
		return false
	end
end

------------------------------------------------------------------------------------------
-- ダイレクトコマンドのリクエストフレームを送信して結果ステータスをチェックする
-- 正常にリプライを受信した後、リプライデータ中のステータスバイトが 0x01 の時のみ
-- trueとリプライフレーム中のデータ部分を返しそれ以外は false を返す
------------------------------------------------------------------------------------------
function dxcmd_exec_reply_get(node,frame)
	local dxstat,reply = mgcp_dx_command(node,frame,false,500)
	if dxstat then
		local cmdstat = table.remove(reply,1)
		if cmdstat == 0x01 then
			return true,reply
		else
			return false
		end
	else
		return false
	end
end

-------------------------------------------------------------------------------------
-- ダイレクトコマンドフレーム配列にバイトデータを追加する
-------------------------------------------------------------------------------------
function dxcmd_param_add_byte(tbl,x)
	table.insert(tbl,bit_and(x,0xff))
end

-------------------------------------------------------------------------------------
-- ダイレクトコマンドフレーム配列にワードデータ(2bytes BigEndian)を追加する
-------------------------------------------------------------------------------------
function dxcmd_param_add_word(tbl,x)
	table.insert(tbl,bit_and(bit_rshift(x,8),0xff))
	table.insert(tbl,bit_and(x,0xff))
end

-------------------------------------------------------------------------------------
-- ダイレクトコマンドフレーム配列にワードデータ(4bytes BigEndian)を追加する
-------------------------------------------------------------------------------------
function dxcmd_param_add_long(tbl,x)
	table.insert(tbl,bit_and(bit_rshift(x,24),0xff))
	table.insert(tbl,bit_and(bit_rshift(x,16),0xff))
	table.insert(tbl,bit_and(bit_rshift(x,8),0xff))
	table.insert(tbl,bit_and(x,0xff))
end

-------------------------------------------------------------------------------------
-- ダイレクトコマンドフレーム配列に別の配列データを追加する
-------------------------------------------------------------------------------------
function dxcmd_param_append(tbl,atbl)
	for i,v in ipairs(atbl) do
		table.insert(tbl,v)
	end
end
