
----------------------------------------------------------------------------------------------------------
-- FONTX2 形式のフォントを扱うためのライブラリ関数
--
-- ChangeLog:
--
-- 2022/10/1    _ascii5x8_font 追加
--
-- 2020/12/9    _free_pixel_font 追加
--				ロード済みのフォント名リストをグローバル共有文字列リスト"$FONTX_LIST"に保存するようにした
--
-- 2019/12/13	fontx_8x8_get() を廃止して代わりに　shmem_get_fontx() ライブラリ関数を使用するようにした
--				fontx_init() に _ayu_font/AYU.FNT と _shinonome_font/SHINO.FNT  のロードを追加
--
-- abs_agent	All Blue System
----------------------------------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- フォントデータファイルをロードしてグローバル共有メモリに保存する
-- ロード済みのフォント名はグローバル共有文字列リスト"$FONTX_LIST" に保存する
--------------------------------------------------------------------------------
function fontx_init()
	local listname = "$FONTX_LIST"
	local stat,mod_stat,config = service_module_status()
	local script_root = config["ScriptFolder"]
	local program_root = config["ProgramFolder"]
	local agent_shmem_path = program_root .. "/agent_shmem"

	font_tbl = {
		-- 美咲フォント (全角 8x8 Shift-JIS)
		{"MISAKI_FONTX",		"/preload/013_FONTX/_misaki_font/MISAKI.FNT"},	

		-- AYUフォント (全角 20x20 Shift-JIS)
		{"AYU_FONTX",			"/preload/013_FONTX/_ayu_font/AYU.FNT"},	

		-- AYUフォント (半角 10x20 ASCII)
		{"E_AYU_FONTX",			"/preload/013_FONTX/_ayu_font/E_AYU.FNT"},	

		-- 東雲フォント (全角 12x12 Shift-JIS)
		{"SHINONOME12_FONTX",	"/preload/013_FONTX/_shinonome_font/SHINO12.FNT"},	

		-- 東雲フォント (全角 14x14 Shift-JIS)
		{"SHINONOME14_FONTX",	"/preload/013_FONTX/_shinonome_font/SHINO14.FNT"},	

		-- 東雲フォント (全角 16x16 Shift-JIS)
		{"SHINONOME_FONTX",		"/preload/013_FONTX/_shinonome_font/SHINO.FNT"},	

		-- 東雲フォント (半角 6x12 ASCII)
		{"E_SHINONOME12_FONTX",	"/preload/013_FONTX/_shinonome_font/E_SHINO12.FNT"},	

		-- 東雲フォント (半角 7x14 ASCII)
		{"E_SHINONOME14_FONTX",	"/preload/013_FONTX/_shinonome_font/E_SHINO14.FNT"},	

		-- 東雲フォント (半角 8x16 ASCII)
		{"E_SHINONOME_FONTX",	"/preload/013_FONTX/_shinonome_font/E_SHINO.FNT"},	

		-- FreePixel (8x16 ASCII)
		{"FREEPIXEL_FONTX",		"/preload/013_FONTX/_free_pixel_font/FREEPXL.FNT"},	

		-- ASCII (5x8 ASCII)
		{"ASCII58_FONTX",	"/preload/013_FONTX/_ascii5x8_font/ASC58.FNT"},	

	}

	stat_check(clear_shared_strlist(listname))

	for i,v in ipairs(font_tbl) do
		local font_name = v[1]
		local font_path = script_root .. v[2]
		local store_cmd = agent_shmem_path .. " -c " .. font_name .. " -l " .. font_path
		log_msg(store_cmd,"FONTX_LIB")
		stat_check(os.execute(store_cmd))
		stat_check(add_shared_strlist(listname,font_name))
	end
end

function fontx_try_init() -- まだ初期化されていないときにのみ初期化を行う
	local stat,flag = get_shared_data("FONTX_INITIALIZED") -- 厳密な排他は行わないが充分である
	if flag ~= "1" then 
		set_shared_data("FONTX_INITIALIZED","1")
		fontx_init()
	end
end

-------------------------------------------------------------------------------------------------
-- 文字コードが格納された配列中の ASCII 文字を同じ字形の全角文字コード(Shift-JIS) 文字に変換する
-------------------------------------------------------------------------------------------------
function fontx_ascii_to_zenkaku(code_arr)
	for i,c in ipairs(code_arr) do
		if (c == 0x20) then code_arr[i] = 0x8140 end -- ' '
		if (c == 0x21) then code_arr[i] = 0x8149 end -- '!'
		if (c == 0x22) then code_arr[i] = 0x818D end -- '"'
		if (c == 0x23) then code_arr[i] = 0x8194 end -- '#'
		if (c == 0x24) then code_arr[i] = 0x8190 end -- '$'
		if (c == 0x25) then code_arr[i] = 0x8193 end -- '%'
		if (c == 0x26) then code_arr[i] = 0x8195 end -- '&'
		if (c == 0x27) then code_arr[i] = 0x818C end -- '''
		if (c == 0x28) then code_arr[i] = 0x8169 end -- '('
		if (c == 0x29) then code_arr[i] = 0x816A end -- ')'
		if (c == 0x2A) then code_arr[i] = 0x8196 end -- '*'
		if (c == 0x2B) then code_arr[i] = 0x817B end -- '+'
		if (c == 0x2C) then code_arr[i] = 0x8143 end -- ','
		if (c == 0x2D) then code_arr[i] = 0x817C end -- '-'
		if (c == 0x2E) then code_arr[i] = 0x8144 end -- '.'
		if (c == 0x2F) then code_arr[i] = 0x815E end -- '/'

		if (c >= 0x30) and (c <= 0x39) then code_arr[i] = 0x824F + (c - 0x30) end -- '0'..'9'

		if (c == 0x3A) then code_arr[i] = 0x8146 end -- ':'
		if (c == 0x3B) then code_arr[i] = 0x8147 end -- ';'
		if (c == 0x3C) then code_arr[i] = 0x8183 end -- '<'
		if (c == 0x3D) then code_arr[i] = 0x8181 end -- '='
		if (c == 0x3E) then code_arr[i] = 0x8184 end -- '>'
		if (c == 0x3F) then code_arr[i] = 0x8148 end -- '?'

		if (c == 0x40) then code_arr[i] = 0x8197 end -- '@'

		if (c >= 0x41) and (c <= 0x5A) then code_arr[i] = 0x8260 + (c - 0x41) end -- 'A'..'Z'

		if (c == 0x5B) then code_arr[i] = 0x816D end -- '['
		if (c == 0x5C) then code_arr[i] = 0x815F end -- '\'
		if (c == 0x5D) then code_arr[i] = 0x816E end -- ']'
		if (c == 0x5E) then code_arr[i] = 0x814F end -- '^'
		if (c == 0x5F) then code_arr[i] = 0x8151 end -- '_'

		if (c == 0x60) then code_arr[i] = 0x814C end -- '''

		if (c >= 0x61) and (c <= 0x7A) then code_arr[i] = 0x8281 + (c - 0x61) end -- 'a'..'z'
		
		if (c == 0x7B) then code_arr[i] = 0x816F end -- '{'
		if (c == 0x7C) then code_arr[i] = 0x8162 end -- '|'
		if (c == 0x7D) then code_arr[i] = 0x8170 end -- '}'
		if (c == 0x7E) then code_arr[i] = 0x8160 end -- '~'
	end
end
