
----------------------------------------------------------------------------
-- LCD(ST7789)ディスプレイ出力用ライブラリ関数
--
-- change history:
--
-- ver1.20 2022/6/4  ST7789_display_reload_page() を ST7789_display() に、ST7789_display() を ST7789_display_reset() に変更
--
-- ver1.10 2022/2/21 graphic2 ライブラリ版に移行
--
-- ver1.00 2022/2/4	初版
--
--                                            abs_agent  All Blue System
----------------------------------------------------------------------------

ST7789_CLOCK_DIV = 8 -- SPI clock =  250MHz/<ST7789_CLOCK_DIV>
ST7789_SPI_PORT = 0
ST7789_DC_GPIO = 24
ST7789_RESET_GPIO = 25
ST7789_WIDTH = 240
ST7789_HEIGHT = 240
ST7789_DISP_BUFF_CH = "ST7789DispBuff"
ST7789_PAGE_SIZE = 2 * ST7789_WIDTH * ST7789_HEIGHT
ST7789_PAGE_MAX = 2	-- ライブラリ内で使用可能なページ(フレームバンク)数

---------------------------------------------------------------------------------------
-- 1byte データを LCD に転送する
---------------------------------------------------------------------------------------
function ST7789_data(byte)
	stat_check(raspi_gpio_write(ST7789_DC_GPIO,true)) -- set DC high for data
	stat_check(raspi_spi_write_byte(byte))
end

---------------------------------------------------------------------------------------
-- 1byte コマンドを LCD に転送する
---------------------------------------------------------------------------------------
function ST7789_cmd(byte)
	stat_check(raspi_gpio_write(ST7789_DC_GPIO,false)) -- set DC low for command
	stat_check(raspi_spi_write_byte(byte))
end

---------------------------------------------------------------------------------------
-- 転送対象とするディスプレイバッファのデータサイズを設定する(1page 分のデータ)
---------------------------------------------------------------------------------------
function ST7789_setup_window()
	local x0 = 0
	local y0 = 0
	local x1 = ST7789_WIDTH - 1
	local y1 = ST7789_HEIGHT - 1
	ST7789_cmd(0x2A)	-- column addr set
	ST7789_data(bit_rshift(x0,8))
	ST7789_data(bit_and(x0,0xff))
	ST7789_data(bit_rshift(x1,8))
	ST7789_data(bit_and(x1,0xff))
	ST7789_cmd(0x2B)	-- row addr set
	ST7789_data(bit_rshift(y0,8))
	ST7789_data(bit_and(y0,0xff))
	ST7789_data(bit_rshift(y1,8))
	ST7789_data(bit_and(y1,0xff))
	ST7789_cmd(0x2C)	-- write to RAM
	stat_check(raspi_gpio_write(ST7789_DC_GPIO,true)) -- set DC high for data
end

-----------------------------------------------------------------------------------------
-- ディスプレイバッファ中の指定ページ page の内容を LCD RAM に書き込む
-- 表示後に ptn に指定したカラー値またはバイト値の配列でディスプレイバッファを初期化する
-----------------------------------------------------------------------------------------
function ST7789_display_reset(page,ptn)
	local p = 0
	if page then p = page end
	if p >= ST7789_PAGE_MAX then error() end
	-- setup drawing area
	ST7789_setup_window()
	-- draw LCD
	if ptn then
		if type(ptn) == "table" then -- reload background image 
			stat_check(raspi_spi_write_shmem(ST7789_SPI_PORT,ST7789_DISP_BUFF_CH,p * ST7789_PAGE_SIZE,ST7789_PAGE_SIZE,ptn))
		else -- reload color pattern
			local fill_arr = {}
			table.insert(fill_arr,bit_rshift(bit_and(ptn,0xff00),8))
			table.insert(fill_arr,bit_and(ptn,0xff))
			stat_check(raspi_spi_write_shmem(ST7789_SPI_PORT,ST7789_DISP_BUFF_CH,p * ST7789_PAGE_SIZE,ST7789_PAGE_SIZE,fill_arr))
		end
	else
		stat_check(raspi_spi_write_shmem(ST7789_SPI_PORT,ST7789_DISP_BUFF_CH,p * ST7789_PAGE_SIZE,ST7789_PAGE_SIZE))
	end
end

---------------------------------------------------------------------------------------
-- ディスプレイバッファ中の指定ページ page の内容を LCD RAM に書き込む
-- 表示後に reload_page に指定した内容で page パラメータで指定したメモリ値を上書きする
---------------------------------------------------------------------------------------
function ST7789_display(page,reload_page)
	local p = 0
	if page then p = page end
	if p >= ST7789_PAGE_MAX then error() end
	-- setup drawing area
	ST7789_setup_window()
	-- draw LCD
	if reload_page then 
		if reload_page >= ST7789_PAGE_MAX then error() end
		-- draw LCD and reload page
		stat_check(raspi_spi_write_shmem(ST7789_SPI_PORT,ST7789_DISP_BUFF_CH,p * ST7789_PAGE_SIZE,ST7789_PAGE_SIZE,reload_page * ST7789_PAGE_SIZE))
	else
		-- draw LCD
		stat_check(raspi_spi_write_shmem(ST7789_SPI_PORT,ST7789_DISP_BUFF_CH,p * ST7789_PAGE_SIZE,ST7789_PAGE_SIZE))
	end
end

---------------------------------------------------------------------------------------
-- LCD デバイスの初期化
---------------------------------------------------------------------------------------
function ST7789_init()
	---------------------------
	-- setup control pins
	---------------------------
	stat_check(raspi_gpio_config(ST7789_DC_GPIO,"output","off"))
	stat_check(raspi_gpio_config(ST7789_RESET_GPIO,"output","off"))
	---------------------------
	-- setup SPI port
	---------------------------
	stat_check(raspi_spi_config(ST7789_SPI_PORT,true,"mode3",ST7789_CLOCK_DIV,"cs0","low"))
	---------------------------
	-- hardware reset
	---------------------------
	stat_check(raspi_gpio_write(ST7789_RESET_GPIO,true))
	wait_time(120)
	stat_check(raspi_gpio_write(ST7789_RESET_GPIO,false))
	wait_time(120)
	stat_check(raspi_gpio_write(ST7789_RESET_GPIO,true))
	wait_time(120)
	---------------------------
	-- initialize the display
	---------------------------
	ST7789_cmd(0x01)			-- Software reset
	wait_time(120)

	ST7789_cmd(0x36)			-- Memory Data Access Control
	ST7789_data(0xB0)
	--ST7789_data(0x70)
	wait_time(10)

	ST7789_cmd(0x37)			-- Vertical Scroll Start Address of RAM
	ST7789_data(0x0)
--	ST7789_data(320 - ST7789_WIDTH)
	ST7789_data(ST7789_WIDTH)

	ST7789_cmd(0xB2)			-- Porch Setting
	ST7789_data(0x0C)
	ST7789_data(0x0C)
	ST7789_data(0x00)
	ST7789_data(0x33)
	ST7789_data(0x33)

	ST7789_cmd(0x3A)			-- Interface Pixel Format
	ST7789_data(0x05)			-- 16bit/pixel RGB-5-6-5-bit (2bytes)

	ST7789_cmd(0xB7)			-- Gate Control
	ST7789_data(0x14)

	ST7789_cmd(0xBB)			-- VCOMS Setting
	ST7789_data(0x37)

	ST7789_cmd(0xC0)			-- LCM Control
	ST7789_data(0x2C)

	ST7789_cmd(0xC2)			-- VDV and VRH Command Enable
	ST7789_data(0x01)

	ST7789_cmd(0xC3)			-- VRH Set
	ST7789_data(0x12)

	ST7789_cmd(0xC4)			-- VDV Set			
	ST7789_data(0x20)

	ST7789_cmd(0xD0)			-- Power Control 1
	ST7789_data(0xA4)
	ST7789_data(0xA1)

	ST7789_cmd(0xC6)			-- Frame Rate Control in Normal Mode
	ST7789_data(0x0F)

	ST7789_cmd(0xE0)			-- Positive Voltage Gamma Control
	ST7789_data(0xD0)
	ST7789_data(0x04)
	ST7789_data(0x0D)
	ST7789_data(0x11)
	ST7789_data(0x13)
	ST7789_data(0x2B)
	ST7789_data(0x3F)
	ST7789_data(0x54)
	ST7789_data(0x4C)
	ST7789_data(0x18)
	ST7789_data(0x0D)
	ST7789_data(0x0B)
	ST7789_data(0x1F)
	ST7789_data(0x23)

	ST7789_cmd(0xE1)			-- Negative Voltage Gamma Control
	ST7789_data(0xD0)
	ST7789_data(0x04)
	ST7789_data(0x0C)
	ST7789_data(0x11)
	ST7789_data(0x13)
	ST7789_data(0x2C)
	ST7789_data(0x3F)
	ST7789_data(0x44)
	ST7789_data(0x51)
	ST7789_data(0x2F)
	ST7789_data(0x1F)
	ST7789_data(0x1F)
	ST7789_data(0x20)
	ST7789_data(0x23)

--	ST7789_cmd(0x20)			-- Display Inversion Off
	ST7789_cmd(0x21)			-- Display Inversion On

	ST7789_cmd(0x11)			-- Sleep Out

	ST7789_cmd(0x29) 			-- Display On
	wait_time(100)

	------------------------------
	-- ディスプレイバッファ初期化
	------------------------------
	graphic2_init(ST7789_DISP_BUFF_CH,ST7789_WIDTH,ST7789_HEIGHT,ST7789_PAGE_MAX,16)
end

function ST7789_try_init() -- まだデバイスが初期化されていないときにのみ初期化を行う
	local key = "ST7789_" .. "SPI" .. tostring(ST7789_SPI_PORT) .. "_INITIALIZED"
	local stat,flag = get_shared_data(key) -- 厳密な排他は行わないが充分である
	if flag ~= "1" then 
		set_shared_data(key,"1")
		ST7789_init()
		stat_check(graphic2_clear(ST7789_DISP_BUFF_CH))
	end
end
