
----------------------------------------------------------------------------
-- Pimoroni Display HAT Mini LCD(ST7789)ディスプレイ出力用ライブラリ関数
--
-- change history:
--
-- ver1.0  2023/9/23 初版, ST7789_LIB.lua から移植
--
--                                            abs_agent  All Blue System
----------------------------------------------------------------------------

DISPHATMINI_CLOCK_DIV = 6 -- SPI clock =  250MHz/<DISPHATMINI_CLOCK_DIV>
DISPHATMINI_SPI_PORT = 0
DISPHATMINI_DC_GPIO = 9
DISPHATMINI_CS1_GPIO = 7	-- Soc 内蔵SPIモジュールによる自動アサートを使用しないで、GPIOポートを直接ライブラリから操作する。

DISPHATMINI_WIDTH = 320
DISPHATMINI_HEIGHT = 240
DISPHATMINI_DISP_BUFF_CH = "DispHATMiniBuff"
DISPHATMINI_PAGE_SIZE = 2 * DISPHATMINI_WIDTH * DISPHATMINI_HEIGHT
DISPHATMINI_PAGE_MAX = 2	-- ライブラリ内で使用可能なページ(フレームバンク)数

DISPHATMINI_LED_R = 17
DISPHATMINI_LED_G = 27
DISPHATMINI_LED_B = 22

DISPHATMINI_BUTTON_A = 5
DISPHATMINI_BUTTON_B = 6
DISPHATMINI_BUTTON_X = 16
DISPHATMINI_BUTTON_Y = 24

DISPHATMINI_BACKLIGHT = 13

---------------------------------------------------------------------------------------
-- LCD ChipSelect disable
---------------------------------------------------------------------------------------
function DISPHATMINI_CS_deassert()
	stat_check(raspi_gpio_write(DISPHATMINI_CS1_GPIO,true))
end

---------------------------------------------------------------------------------------
-- LCD ChipSelect enable
---------------------------------------------------------------------------------------
function DISPHATMINI_CS_assert()
	stat_check(raspi_gpio_write(DISPHATMINI_CS1_GPIO,false))
end

---------------------------------------------------------------------------------------
-- 1byte データを LCD に転送する
---------------------------------------------------------------------------------------
function DISPHATMINI_data(byte)
	stat_check(raspi_gpio_write(DISPHATMINI_DC_GPIO,true)) -- set DC high for data
	stat_check(raspi_spi_write_byte(byte))
end

---------------------------------------------------------------------------------------
-- 1byte コマンドを LCD に転送する
---------------------------------------------------------------------------------------
function DISPHATMINI_cmd(byte)
	stat_check(raspi_gpio_write(DISPHATMINI_DC_GPIO,false)) -- set DC low for command
	stat_check(raspi_spi_write_byte(byte))
end

---------------------------------------------------------------------------------------
-- 転送対象とするディスプレイバッファのデータサイズを設定する(1page 分のデータ)
---------------------------------------------------------------------------------------
function DISPHATMINI_setup_window()
	local x0 = 0
	local y0 = 0
	local x1 = DISPHATMINI_WIDTH - 1
	local y1 = DISPHATMINI_HEIGHT - 1
	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0x2A)	-- column addr set
	DISPHATMINI_data(bit_rshift(x0,8))
	DISPHATMINI_data(bit_and(x0,0xff))
	DISPHATMINI_data(bit_rshift(x1,8))
	DISPHATMINI_data(bit_and(x1,0xff))
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0x2B)	-- row addr set
	DISPHATMINI_data(bit_rshift(y0,8))
	DISPHATMINI_data(bit_and(y0,0xff))
	DISPHATMINI_data(bit_rshift(y1,8))
	DISPHATMINI_data(bit_and(y1,0xff))
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0x2C)	-- write to RAM

	stat_check(raspi_gpio_write(DISPHATMINI_DC_GPIO,true)) -- set DC high for data
end

-----------------------------------------------------------------------------------------
-- ディスプレイバッファ中の指定ページ page の内容を LCD RAM に書き込む
-- 表示後に ptn に指定したカラー値またはバイト値の配列でディスプレイバッファを初期化する
-----------------------------------------------------------------------------------------
function DISPHATMINI_display_reset(page,ptn)
	local p = 0
	if page then p = page end
	if p >= DISPHATMINI_PAGE_MAX then error() end
	-- setup drawing area
	DISPHATMINI_setup_window()
	-- draw LCD
	if ptn then
		if type(ptn) == "table" then -- reload background image 
			stat_check(raspi_spi_write_shmem(DISPHATMINI_SPI_PORT,DISPHATMINI_DISP_BUFF_CH,p * DISPHATMINI_PAGE_SIZE,DISPHATMINI_PAGE_SIZE,ptn))
		else -- reload color pattern
			local fill_arr = {}
			table.insert(fill_arr,bit_rshift(bit_and(ptn,0xff00),8))
			table.insert(fill_arr,bit_and(ptn,0xff))
			stat_check(raspi_spi_write_shmem(DISPHATMINI_SPI_PORT,DISPHATMINI_DISP_BUFF_CH,p * DISPHATMINI_PAGE_SIZE,DISPHATMINI_PAGE_SIZE,fill_arr))
		end
	else
		stat_check(raspi_spi_write_shmem(DISPHATMINI_SPI_PORT,DISPHATMINI_DISP_BUFF_CH,p * DISPHATMINI_PAGE_SIZE,DISPHATMINI_PAGE_SIZE))
	end
	DISPHATMINI_CS_deassert()
end

---------------------------------------------------------------------------------------
-- ディスプレイバッファ中の指定ページ page の内容を LCD RAM に書き込む
-- 表示後に reload_page に指定した内容で page パラメータで指定したメモリ値を上書きする
---------------------------------------------------------------------------------------
function DISPHATMINI_display(page,reload_page)
	local p = 0
	if page then p = page end
	if p >= DISPHATMINI_PAGE_MAX then error() end
	-- setup drawing area
	DISPHATMINI_setup_window()
	-- draw LCD
	if reload_page then 
		if reload_page >= DISPHATMINI_PAGE_MAX then error() end
		-- draw LCD and reload page
		stat_check(raspi_spi_write_shmem(DISPHATMINI_SPI_PORT,DISPHATMINI_DISP_BUFF_CH,p * DISPHATMINI_PAGE_SIZE,DISPHATMINI_PAGE_SIZE,reload_page * DISPHATMINI_PAGE_SIZE))
	else
		-- draw LCD
		stat_check(raspi_spi_write_shmem(DISPHATMINI_SPI_PORT,DISPHATMINI_DISP_BUFF_CH,p * DISPHATMINI_PAGE_SIZE,DISPHATMINI_PAGE_SIZE))
	end
	DISPHATMINI_CS_deassert()
end

---------------------------------------------------------------------------------------
-- LCD デバイスの初期化
---------------------------------------------------------------------------------------
function DISPHATMINI_init()

	---------------------------
	-- setup GPIO
	---------------------------

	-- RGB LED
	stat_check(raspi_gpio_config(DISPHATMINI_LED_R,"output","off"))
	stat_check(raspi_gpio_config(DISPHATMINI_LED_G,"output","off"))
	stat_check(raspi_gpio_config(DISPHATMINI_LED_B,"output","off"))
	
	stat_check(raspi_gpio_write(DISPHATMINI_LED_R,true)) -- OFF
	stat_check(raspi_gpio_write(DISPHATMINI_LED_G,true)) -- OFF
	stat_check(raspi_gpio_write(DISPHATMINI_LED_B,true)) -- OFF

	-- SW
	stat_check(raspi_gpio_config(DISPHATMINI_BUTTON_A,"input","pullup"))
	stat_check(raspi_gpio_config(DISPHATMINI_BUTTON_B,"input","pullup"))
	stat_check(raspi_gpio_config(DISPHATMINI_BUTTON_X,"input","pullup"))
	stat_check(raspi_gpio_config(DISPHATMINI_BUTTON_Y,"input","pullup"))
	
	stat_check(raspi_change_detect(DISPHATMINI_BUTTON_A,true))
	stat_check(raspi_change_detect(DISPHATMINI_BUTTON_B,true))
	stat_check(raspi_change_detect(DISPHATMINI_BUTTON_X,true))
	stat_check(raspi_change_detect(DISPHATMINI_BUTTON_Y,true))

	-- LCD Backlight
	stat_check(raspi_gpio_config(DISPHATMINI_BACKLIGHT,"output","off"))
	stat_check(raspi_gpio_write(DISPHATMINI_BACKLIGHT,true)) -- ON

	----------------------------------
	-- setup SPI port & control pins
	----------------------------------
	stat_check(raspi_spi_config(DISPHATMINI_SPI_PORT,true,"mode0",DISPHATMINI_CLOCK_DIV,"reserved","low")) -- SoC 内蔵の CS 機能を無効にする
	stat_check(raspi_gpio_config(DISPHATMINI_CS1_GPIO,"output","off"))	-- raspi_spi_config() の SPI#0-CE をオーバーライドする
	stat_check(raspi_gpio_write(DISPHATMINI_CS1_GPIO,true)) 			-- Deassert Chip Select
	stat_check(raspi_gpio_config(DISPHATMINI_DC_GPIO,"output","off"))	-- raspi_spi_config() の SPI#0-MISO をオーバーライドする

	---------------------------
	-- initialize the display
	---------------------------
	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0x01)			-- Software reset
	DISPHATMINI_CS_deassert()
	wait_time(150)

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0x36)			-- Memory Data Access Control
	DISPHATMINI_data(0xB0)
	DISPHATMINI_CS_deassert()
	wait_time(10)

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0xB2)			-- Frame rate ctrl - idle mode
	DISPHATMINI_data(0x0C)
	DISPHATMINI_data(0x0C)
	DISPHATMINI_data(0x00)
	DISPHATMINI_data(0x33)
	DISPHATMINI_data(0x33)
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0x3A)			-- Interface Pixel Format
	DISPHATMINI_data(0x05)			-- 16bit/pixel RGB-5-6-5-bit (2bytes)
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0xB7)			-- Gate Control
	DISPHATMINI_data(0x14)
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0xBB)			-- VCOMS Setting
	DISPHATMINI_data(0x37)
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0xC0)			-- LCM Control
	DISPHATMINI_data(0x2C)
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0xC2)			-- VDV and VRH Command Enable
	DISPHATMINI_data(0x01)
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0xC3)			-- VRH Set
	DISPHATMINI_data(0x12)
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0xC4)			-- VDV Set			
	DISPHATMINI_data(0x20)
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0xD0)			-- Power Control 1
	DISPHATMINI_data(0xA4)
	DISPHATMINI_data(0xA1)
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0xC6)			-- Frame Rate Control in Normal Mode
	DISPHATMINI_data(0x0F)
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0xE0)			-- Positive Voltage Gamma Control
	DISPHATMINI_data(0xD0)
	DISPHATMINI_data(0x04)
	DISPHATMINI_data(0x0D)
	DISPHATMINI_data(0x11)
	DISPHATMINI_data(0x13)
	DISPHATMINI_data(0x2B)
	DISPHATMINI_data(0x3F)
	DISPHATMINI_data(0x54)
	DISPHATMINI_data(0x4C)
	DISPHATMINI_data(0x18)
	DISPHATMINI_data(0x0D)
	DISPHATMINI_data(0x0B)
	DISPHATMINI_data(0x1F)
	DISPHATMINI_data(0x23)
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0xE1)			-- Negative Voltage Gamma Control
	DISPHATMINI_data(0xD0)
	DISPHATMINI_data(0x04)
	DISPHATMINI_data(0x0C)
	DISPHATMINI_data(0x11)
	DISPHATMINI_data(0x13)
	DISPHATMINI_data(0x2C)
	DISPHATMINI_data(0x3F)
	DISPHATMINI_data(0x44)
	DISPHATMINI_data(0x51)
	DISPHATMINI_data(0x2F)
	DISPHATMINI_data(0x1F)
	DISPHATMINI_data(0x1F)
	DISPHATMINI_data(0x20)
	DISPHATMINI_data(0x23)
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0x21)			-- Display Inversion On
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0x11)			-- Sleep Out
	DISPHATMINI_CS_deassert()

	DISPHATMINI_CS_assert()
	DISPHATMINI_cmd(0x29) 			-- Display On
	DISPHATMINI_CS_deassert()
	wait_time(100)

	------------------------------
	-- ディスプレイバッファ初期化
	------------------------------
	graphic2_init(DISPHATMINI_DISP_BUFF_CH,DISPHATMINI_WIDTH,DISPHATMINI_HEIGHT,DISPHATMINI_PAGE_MAX,16)
end

function DISPHATMINI_try_init() -- まだデバイスが初期化されていないときにのみ初期化を行う
	local key = "DISPHATMINI_" .. "SPI" .. tostring(DISPHATMINI_SPI_PORT) .. "_INITIALIZED"
	local stat,flag = get_shared_data(key) -- 厳密な排他は行わないが充分である
	if flag ~= "1" then 
		set_shared_data(key,"1")
		DISPHATMINI_init()
		stat_check(graphic2_clear(DISPHATMINI_DISP_BUFF_CH))
	end
end
