
--[[
****************************************************************************************************************

●機能概要

MGCP ファームウエアを搭載した M5Stack/M5Stick-C デバイスの LCD操作ライブラリ

ver1.6 2021/5/30	半角・全角混在時のスクリーン幅チェックのバグ修正

ver1.5 2021/2/6		自動改行時のスクリーン幅チェックのバグ修正

ver1.4 2021/1/4		str_to_tbl2() コール時に SJIS フォントに無い文字を '?' に変更するようにした

ver1.3 2020/12/24	mgcp_lcd_jprint() で自動的に全角と半角 ASCII フォントを切り替えて表示するように変更

ver1.2 2020/8/25	mgcp_lcd_jprint() の scr_w, scr_h のデフォルト値をデバイスタイプから取得するように変更

ver1.1 2020/7/6		図形描画関数追加, mgcp_command() コール時の timeout を 10(sec) に変更

ver1.0 新規作成

copyright(c) 2021 All Blue System

****************************************************************************************************************
]]


--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) の LCD に ASCII文字列を表示する。

日本語を表示する場合には　mgcp_lcd_jprint() または mgcp_lcd_jprintln() を使用してください。

関数定義
		mgcp_lcd_print(node,str,x0,y0,wrap,cls,font,fg,bg)

パラメータ
		node:string			MGCP ノード名
		str:string			ASCII文字列。
		x0:number(opt)		表示開始座標(X軸) 
							default: wrap が trueの場合 0,false の場合は str を画面中央に表示する座標	
		y0:number(opt)		表示開始座標(Y軸)
							default: wrap が trueの場合 0,false の場合は str を画面中央に表示する座標	
		wrap:boolean(opt)	長い文字列を自動的に折り返す default: false
							折り返す場合に、縦方向のフォントサイズの 1/5 に相当するピクセル分を空ける。
		cls:boolean(opt)	文字列描画前に画面をクリアする default: false
		font:string(opt)	フォント番号 0..9 の整数を指定 default: 以前に設定した値またはデバイス初期値の 2
		fg:number(opt)		文字色のカラーコードを指定する		default:15 (WHITE)
		bg:number(opt)		背景色のカラーコードを指定する		default:0  (BLACK)

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function mgcp_lcd_print(node,str,x0,y0,wrap,cls,font,fg,bg)
	if not fg then fg = 15 end
	if not bg then bg = 0 end
	if not wrap then wrap = false end
	if not cls then cls = false end
	local mgcp_cmd = '{"command":"lcd_print"'
	if cls then
		mgcp_cmd = mgcp_cmd .. ',"clear":' .. tostring(bg)
	end
	mgcp_cmd = mgcp_cmd .. ',"fg":' .. tostring(fg)
	mgcp_cmd = mgcp_cmd .. ',"bg":' .. tostring(bg)
	if wrap then
		mgcp_cmd = mgcp_cmd .. ',"wrap":1'
	end
	if x0 then
		mgcp_cmd = mgcp_cmd .. ',"x":' .. tostring(x0)
	end
	if y0 then
		mgcp_cmd = mgcp_cmd .. ',"y":' .. tostring(y0)
	end
	if font then
		mgcp_cmd = mgcp_cmd .. ',"font":' .. tostring(font)
	end
	mgcp_cmd = mgcp_cmd .. ',"text":"' .. str .. '"}'
	local mstat,reply = mgcp_command(node,mgcp_cmd,false,10000) -- timeout 10(sec)
	if mstat then
		local rpl_json = g_json.decode(reply)
		if rpl_json.result == "success" then
			return true,rpl_json.level
		else
			return false
		end
	else
		return false
	end
end


--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) の LCD に日本語文字列を表示する。


関数定義
		mgcp_lcd_jprint(node,str,x0,y0,wrap,cls,font,scr_w,scr_h,split,size,fg,bg)

パラメータ
		node:string			MGCP ノード名
		str:string			日本語文字列。ASCII 文字は同じ字形の全角文字に自動変換される。
							各文字のフォントデータが見つからなかった場合にはその文字は表示しない。
		x0:number(opt)		表示開始座標(X軸) default: 0
		y0:number(opt)		表示開始座標(Y軸) default: 0
		wrap:boolean(opt)	長い文字列を自動的に折り返す default: false
							折り返す時に、縦方向のフォントサイズの 1/5 に相当するピクセル分を空ける。
		cls:boolean(opt)	文字列描画前に画面をクリアする default: false
		font:string(opt)	グローバル共有エリアにロード済みの日本語フォントのチャンネル名 default: "SHINONOME_FONTX"
		scr_w:number(opt)	LCD 表示器の横サイズ default:160
		scr_h:number(opt)	LCD 表示器の縦サイズ default:80
		split:number(opt)	指定した文字数ごとに、複数の MGCP コマンドに分割して文字を表示する	default:nil
							パラメータ省略時は全ての文字を一度のコマンドで表示する。ただしこの場合 MGCP コマンド
							のリクエスト文字列が 4096bytesを超える場合にエラーが発生する。
		size:number(opt)	文字を size 倍に拡大して表示する	default:1
		fg:number(opt)		文字色のカラーコードを指定する		default:15 (WHITE)
		bg:number(opt)		背景色のカラーコードを指定する		default:0  (BLACK)

リターン値
		stat:boolean	true 成功、false 失敗

グローバル共有変数
		LCD に最後に表示した"文字"の次の表示開始予定座標をグローバル共有変数に保存する。
		前回表示した文字列に続けて表示したい場合に、次に表示開始座標を計算するときに利用できる。
		画面のサイズは考慮していないので折り返し表示が必要な場合は座標値の再計算が必要。

		キー名                    値
		-------------------------+---------------------------------------------------------
		MGCP_LCD_PREV_<node>	  表示した最後の文字の情報 (CSV形式で <x0>,<y0>,<fh>,<size>)
 
		<x0>	      表示した最後の文字の次の開始座標 X軸
		<y0>	      表示した最後の文字の次の開始座標 Y軸
		<fh>	      表示した最後の文字の縦方向のフォントサイズ
		<size>	      表示した最後の文字のフォント拡大倍率(size パラメータに指定したもの)

****************************************************************************************************
]]

----------------------------------------------------------------------------------------------------
-- bmd_cmd_arr[] 配列に保存してある lcd_bitmap コマンドパラメータを展開して MGCP コマンドを実行する
----------------------------------------------------------------------------------------------------
function mgcp_exec_lcd_cmd(node,bmp_cmd_arr,cls,size,fg,bg)
	local mgcp_cmd = '{"command":"lcd_bitmap"'
	if not fg then fg = 15 end
	if not bg then bg = 0 end
	if cls then
		mgcp_cmd = mgcp_cmd .. ',"clear":' .. tostring(bg)
	end
	mgcp_cmd = mgcp_cmd .. ',"fg":' .. tostring(fg)
	mgcp_cmd = mgcp_cmd .. ',"bg":' .. tostring(bg)
	if size and (size > 1) then
		mgcp_cmd = mgcp_cmd .. ',"size":' .. tostring(size)
	end
	if not size then size = 1 end
	mgcp_cmd = mgcp_cmd .. ',"data":' .. g_json.encode(bmp_cmd_arr) .. '}'
	local mstat,reply = mgcp_command(node,mgcp_cmd,false,10000) -- timeout 10(sec)
	if mstat then
		local rpl_json = g_json.decode(reply)
		if rpl_json.result == "success" then
			set_shared_data("MGCP_LCD_PREV_" .. node,
							tostring(bmp_cmd_arr[#bmp_cmd_arr -4] + bmp_cmd_arr[#bmp_cmd_arr -2]) .. ',' .. 
							tostring(bmp_cmd_arr[#bmp_cmd_arr -3])  .. ',' ..
							tostring(bmp_cmd_arr[#bmp_cmd_arr -1]) .. ',' ..
							tostring(size))
			return true
		else
			return false
		end
	else
		return false
	end
end

function mgcp_lcd_jprint(node,str,x0,y0,wrap,cls,font,scr_w,scr_h,split,size,fg,bg)
	fontx_try_init() -- FONTX ライブラリがまだ初期化されていない場合にはフォントをグローバル共有メモリにロードする
	if not x0 then x0 = 0 end
	if not y0 then y0 = 0 end
	if not wrap then wrap = false end
	if not cls then cls = false end
	if (not scr_w) or (not scr_h) then
		if not scr_w then scr_w = 160 end
		if not scr_h then scr_h = 80 end
		local dev_type = stat_check(get_shared_data("$TYPE_" .. node))
		if dev_type == "M5Stack" then
			scr_w = 320
			scr_h = 240
		end
		if dev_type == "M5Stick" then
			scr_w = 160
			scr_h = 80
		end
	end
	if not size then size = 1 end
	-- SJIS 用フォントロード
	if not font then font = "SHINONOME_FONTX" end
	local stat, x = shmem_copy(font,14,2) 		-- x[1]:width, x[2]:height, x[3]:Code 0-ASCII 1-SJISFONTX2 形式のフォントサイズ部分を取得する
	if not stat then return false end
	local font_w = x[1]
	local font_h = x[2]
	-- ASCII 用のフォントがあればロード
	local e_font_w,e_font_h
	local e_font = "E_" .. font
	if find_shared_strlist("$FONTX_LIST",e_font) then
		local stat,e_font_info = shmem_copy(e_font,14,3)
		if stat and (e_font_info[3] == 0) then
			use_e_font = true
			e_font_w = e_font_info[1]
			e_font_h = e_font_info[2]
		end
	else
		e_font = ""
	end
	local sp_h = math.floor((font_h * size) / 5)	-- 縦方向の文字間隔を計算
	local cstat,ch_arr = str_to_tbl2(str,2,0x3f) 	-- UTF-8文字列を 1文字毎の配列に変換して且つ、文字コードをShift_JIS に変換
	if (not cstat) or (#ch_arr == 0) then return false end
	if e_font == "" then
		fontx_ascii_to_zenkaku(ch_arr)				-- ASCII コード部分のフォントが無いので同じ字形の Shift_JIS 全角文字に変換
	end
	local bmp_cmd = {}
	local ch_idx = 0
	local cmd_stat
	if split and (#ch_arr > split) then
		---------------------------------------------------------------------------------------------------------
		-- ** split mode **  split パラメータで指定した文字数毎に lcd_bitmap コマンドを複数回コールする。
		---------------------------------------------------------------------------------------------------------
		local prev_split_idx = 0
		local split_idx						-- lcd_bitmap コマンドを複数回コールするときに、何回目のコールになるかを示す
		local first = true						-- 最初の lcd_bitmap コール時に画面クリア関連のフラグとして使用
		for k,v in ipairs(ch_arr) do
			split_idx = math.modf((k + split - 1)/split)
			if (prev_split_idx ~= split_idx) and (split_idx > 1) then
				cmd_stat = mgcp_exec_lcd_cmd(node,bmp_cmd,cls and first,size,fg,bg)
				if not cmd_stat then return false end
				first = false
				bmp_cmd = {}
				ch_idx = 0
			end
		
			if v < 0xff then
				local gstat, fdata = shmem_get_fontx(e_font,v)	-- ASCIIフォントデータ取得
				if not gstat then goto NEXT_CHAR_SPLIT end	-- フォントデータが無かった場合にはその文字をスキップ
				-- table.insert() で HEX文字列を挿入するとエスケープキャラクタが入るので、配列への代入ステートメント使用
				if (x0 + (e_font_w * size)) > scr_w then		-- このまま表示すると画面横方向にはみ出しそう？
					if not wrap then break end				-- wrap 指定が無いならそこで表示終了
					x0 = 0										-- wrap 指定ならば縦にずらして改行を演出
					y0 = y0 + sp_h + (e_font_h * size)
				end
				if (y0 + (e_font_h * size)) >= scr_h then break end		-- 表示すると画面縦方向にはみ出る見込みなら表示終了
				ch_idx = ch_idx + 1
				bmp_cmd[(ch_idx - 1) * 5 + 1] = x0
				bmp_cmd[(ch_idx - 1) * 5 + 2] = y0
				bmp_cmd[(ch_idx - 1) * 5 + 3] = e_font_w
				bmp_cmd[(ch_idx - 1) * 5 + 4] = e_font_h
				bmp_cmd[(ch_idx - 1) * 5 + 5] = tbl_to_hex(fdata)
				x0 = x0 + (e_font_w * size)
			else
				local gstat, fdata = shmem_get_fontx(font,v)	-- SJISフォントデータ取得
				if not gstat then goto NEXT_CHAR_SPLIT end
				if (x0 + (font_w * size)) > scr_w then
					if not wrap then break end
					x0 = 0
					y0 = y0 + sp_h + (font_h * size)
				end
				if (y0 + (font_h * size)) >= scr_h then break end
				ch_idx = ch_idx + 1
				bmp_cmd[(ch_idx - 1) * 5 + 1] = x0
				bmp_cmd[(ch_idx - 1) * 5 + 2] = y0
				bmp_cmd[(ch_idx - 1) * 5 + 3] = font_w
				bmp_cmd[(ch_idx - 1) * 5 + 4] = font_h
				bmp_cmd[(ch_idx - 1) * 5 + 5] = tbl_to_hex(fdata)
				x0 = x0 + (font_w * size)
			end

::NEXT_CHAR_SPLIT::
			if k == #ch_arr then
				cmd_stat = mgcp_exec_lcd_cmd(node,bmp_cmd,cls and first,size,fg,bg)
				if not cmd_stat then return false end
				bmp_cmd = {}
				ch_idx = 0
			end
			prev_split_idx = split_idx
		end
		if #bmp_cmd > 1 then -- 画面はみ出しのため途中で表示対象の文字追加を中止したときに、表示可能分の未転送文字列を描画
			cmd_stat = mgcp_exec_lcd_cmd(node,bmp_cmd,cls and first,size,fg,bg)
		end
		return cmd_stat
	else
		---------------------------------------------------------------------------------------------------------
		-- ** whole mode **  文字列全体を一回の lcd_bitmap コマンドで送信する。
		---------------------------------------------------------------------------------------------------------
		for k,v in ipairs(ch_arr) do

			if v < 0xff then
				local gstat, fdata = shmem_get_fontx(e_font,v)
				if not gstat then goto NEXT_CHAR_WHOLE end
				if (x0 + (e_font_w * size)) > scr_w then
					if not wrap then break end
					x0 = 0
					y0 = y0 + sp_h + (e_font_h * size)
				end
				if (y0 + (e_font_h * size)) >= scr_h then break end
				ch_idx = ch_idx + 1
				bmp_cmd[(ch_idx - 1) * 5 + 1] = x0
				bmp_cmd[(ch_idx - 1) * 5 + 2] = y0
				bmp_cmd[(ch_idx - 1) * 5 + 3] = e_font_w
				bmp_cmd[(ch_idx - 1) * 5 + 4] = e_font_h
				bmp_cmd[(ch_idx - 1) * 5 + 5] = tbl_to_hex(fdata)
				x0 = x0 + (e_font_w * size)
			else
				local gstat, fdata = shmem_get_fontx(font,v)
				if not gstat then goto NEXT_CHAR_WHOLE end
				if (x0 + (font_w * size)) > scr_w then
					if not wrap then break end
					x0 = 0
					y0 = y0 + sp_h + (font_h * size)
				end
				if (y0 + (font_h * size)) >= scr_h then break end
				ch_idx = ch_idx + 1
				bmp_cmd[(ch_idx - 1) * 5 + 1] = x0
				bmp_cmd[(ch_idx - 1) * 5 + 2] = y0
				bmp_cmd[(ch_idx - 1) * 5 + 3] = font_w
				bmp_cmd[(ch_idx - 1) * 5 + 4] = font_h
				bmp_cmd[(ch_idx - 1) * 5 + 5] = tbl_to_hex(fdata)
				x0 = x0 + (font_w * size)
			end

::NEXT_CHAR_WHOLE::
		end
		return mgcp_exec_lcd_cmd(node,bmp_cmd,cls,size,fg,bg)
	end
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) の LCD に日本語文字列を表示する。
前回 LCD に表示した文字列の終端で改行して画面左端から表示する。
長い文字列は自動的に折り返して表示する。

関数定義
		mgcp_lcd_jprintln(node,str,cls,font,scr_w,scr_h,split,size,fg,bg)

パラメータ
		node:string			MGCP ノード名
		str:string			日本語文字列。ASCII 文字は同じ字形の全角文字に自動変換される。
							各文字のフォントデータが見つからなかった場合にはその文字は表示しない。
		cls:boolean(opt)	文字列描画前に画面をクリアする default: false
		font:string(opt)	グローバル共有エリアにロード済みの日本語フォントのチャンネル名 default: "SHINONOME_FONTX"
		scr_w:number(opt)	LCD 表示器の横サイズ default:160
		scr_h:number(opt)	LCD 表示器の縦サイズ default:80
		split:number(opt)	指定した文字数ごとに、複数の MGCP コマンドに分割して文字を表示する	default:nil
							パラメータ省略時は全ての文字を一度のコマンドで表示する。ただしこの場合 MGCP コマンド
							のリクエスト文字列が 4096bytesを超える場合にエラーが発生する。
		size:number(opt)	文字を size 倍に拡大して表示する	default:1
		fg:number(opt)		文字色のカラーコードを指定する		default:15 (WHITE)
		bg:number(opt)		背景色のカラーコードを指定する		default:0  (BLACK)


リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function mgcp_lcd_jprintln(node,str,cls,font,scr_w,scr_h,split,size,fg,bg)
	if not size then size = 1 end
	local stat, prev_info = get_shared_data("MGCP_LCD_PREV_" .. node)
	local prev_info_arr = csv_to_tbl(prev_info)
	if cls or (#prev_info_arr ~= 4) then 
		return mgcp_lcd_jprint(node,str,0,0,true,cls,font,scr_w,scr_h,split,size,fg,bg)
	end
	local y0 = tonumber(prev_info_arr[2])
	local font_h = tonumber(prev_info_arr[3])
	local prev_size = tonumber(prev_info_arr[4])
	local sp_h = math.floor((font_h * prev_size) / 5)
	y0 = y0 + sp_h + (font_h * prev_size)
	return mgcp_lcd_jprint(node,str,0,y0,true,cls,font,scr_w,scr_h,split,size,fg,bg)
end


----------------------------------------------------------------------------------------------------
-- params[] 配列に保存してある lcd_draw コマンドの図形表示データを展開して MGCP コマンドを実行する
----------------------------------------------------------------------------------------------------
function mgcp_lcd_draw_cmd(node,cmd,params,cls,fill)
	local mgcp_cmd = '{"command":"lcd_draw"'
	if cls then
		mgcp_cmd = mgcp_cmd .. ',"clear":0'
	end
	if fill then
		mgcp_cmd = mgcp_cmd .. ',"fill":1'
	end
	if cmd and (#params > 0) then
		mgcp_cmd = mgcp_cmd .. ',"' .. cmd .. '":' .. g_json.encode(params) .. '}'
	else
		mgcp_cmd = mgcp_cmd .. '}'
	end
	local mstat,reply = mgcp_command(node,mgcp_cmd,false,10000) -- timeout 10(sec)
	if mstat then
		local rpl_json = g_json.decode(reply)
		if rpl_json.result == "success" then
			return true
		else
			return false
		end
	else
		return false
	end
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD を指定した色で塗りつぶす

関数定義
		mgcp_lcd_fill_screen(node,color)

パラメータ
		node:string			MGCP ノード名

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function mgcp_lcd_fill_screen(node,color)
	local mgcp_cmd = '{"command":"lcd_draw","clear":' .. tostring(color) .. '}'
	local mstat,reply = mgcp_command(node,mgcp_cmd,false,10000) -- timeout 10(sec)
	if mstat then
		local rpl_json = g_json.decode(reply)
		if rpl_json.result == "success" then
			return true
		else
			return false
		end
	else
		return false
	end
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD をクリアする(黒で塗りつぶす)

関数定義
		mgcp_lcd_clear(node)

パラメータ
		node:string			MGCP ノード名

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function mgcp_lcd_clear(node)
	return mgcp_lcd_draw_cmd(node,nil,{},true,nil)
end


--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD に線分を描画する。

関数定義
		mgcp_lcd_line(node,x0,y0,x1,y1[,color][,cls])

パラメータ
		node:string			MGCP ノード名
		x0:number			線分開始座標(X軸) 
		y0:number			線分開始座標(Y軸)
		x1:number			線分終了座標(X軸) 
		y1:number			線分終了座標(Y軸)
		color:number		描画色のカラーコードを指定する		default:15 (WHITE)
		cls:boolean			描画前に画面をクリアする default: false

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function mgcp_lcd_line(node,x0,y0,x1,y1,color,cls)
	if not color then color = 15 end
	return mgcp_lcd_draw_cmd(node,"line",{x0,y0,x1,y1,color},cls,nil)
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD に円を描画する。

関数定義
		mgcp_lcd_circle(node,x,y,r,[,color][,cls][,fill])

パラメータ
		node:string			MGCP ノード名
		x:number			座標(X軸) 
		y:number			座標(Y軸)
		r:number			半径 
		color:number		描画色のカラーコードを指定する		default:15 (WHITE)
		cls:boolean			描画前に画面をクリアする default: false
		fill:boolean		図形内部を塗りつぶす default: false

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function mgcp_lcd_circle(node,x,y,r,color,cls,fill)
	if not color then color = 15 end
	return mgcp_lcd_draw_cmd(node,"circle",{x,y,r,color},cls,fill)
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD に四角を描画する。

関数定義
		mgcp_lcd_rect(node,x,y,w,h [,color][,cls][,fill])

パラメータ
		node:string			MGCP ノード名
		x:number			座標(X軸) 
		y:number			座標(Y軸)
		w:number			四角形の幅
		h:number			四角形の高さ
		color:number		描画色のカラーコードを指定する		default:15 (WHITE)
		cls:boolean			描画前に画面をクリアする default: false
		fill:boolean		図形内部を塗りつぶす default: false

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function mgcp_lcd_rect(node,x,y,w,h,color,cls,fill)
	if not color then color = 15 end
	return mgcp_lcd_draw_cmd(node,"rect",{x,y,w,h,color},cls,fill)
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD に三角を描画する。

関数定義
		mgcp_lcd_triangle(node,x0,y0,x1,y1,x2,y2[,color][,cls])

パラメータ
		node:string			MGCP ノード名
		x0:number			頂点1座標(X軸) 
		y0:number			頂点1座標(Y軸)
		x1:number			頂点2座標(X軸) 
		y1:number			頂点2座標(Y軸)
		x2:number			頂点3座標(X軸) 
		y2:number			頂点3座標(Y軸)
		color:number		描画色のカラーコードを指定する		default:15 (WHITE)
		cls:boolean			描画前に画面をクリアする default: false

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function mgcp_lcd_triangle(node,x0,y0,x1,y1,x2,y2,color,cls,fill)
	if not color then color = 15 end
	return mgcp_lcd_draw_cmd(node,"triangle",{x0,y0,x1,y1,x2,y2,color},cls,fill)
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD に角丸四角を描画する。

関数定義
		mgcp_lcd_round_rect(node,x,y,w,h,r [,color][,cls][,fill])

パラメータ
		node:string			MGCP ノード名
		x:number			座標(X軸) 
		y:number			座標(Y軸)
		w:number			四角形の幅
		h:number			四角形の高さ
		r:number			丸角半径 
		color:number		描画色のカラーコードを指定する		default:15 (WHITE)
		cls:boolean			描画前に画面をクリアする default: false
		fill:boolean		図形内部を塗りつぶす default: false

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function mgcp_lcd_round_rect(node,x,y,w,h,r,color,cls,fill)
	if not color then color = 15 end
	return mgcp_lcd_draw_cmd(node,"round_rect",{x,y,w,h,r,color},cls,fill)
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD に点を描画する。

関数定義
		mgcp_lcd_pixel(node,x,y,[,color][,cls])

パラメータ
		node:string			MGCP ノード名
		x:number			座標(X軸) 
		y:number			座標(Y軸)
		color:number		描画色のカラーコードを指定する		default:15 (WHITE)
		cls:boolean			描画前に画面をクリアする default: false

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function mgcp_lcd_pixel(node,x,y,color,cls)
	if not color then color = 15 end
	return mgcp_lcd_draw_cmd(node,"pixel",{x,y,color},cls,nil)
end

