
--[[
****************************************************************************************************

●機能概要

MGCP ファームウエアを搭載した M5Stack/M5Stick-C デバイスの LCD操作ライブラリ
ダイレクトコマンドモードを使用して、MGCP コマンドに比べて高速に動作する。

2020/08/26 ver1.0 新規作成

copyright(c) 2020 All Blue System

****************************************************************************************************
]]


--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) の LCD に ASCII文字列を表示する。

日本語を表示する場合には　dxcmd_lcd_jprint() または dxcmd_lcd_jprintln() を使用してください。

関数定義
		dxcmd_lcd_print(node,str,x0,y0,wrap,cls,font,fg,bg)

パラメータ
		node:string			MGCP ノード名
		str:string			ASCII文字列。
		x0:number(opt)		表示開始座標(X軸) 
							default: wrap が trueの場合 0,false の場合は str を画面中央に表示する座標	
		y0:number(opt)		表示開始座標(Y軸)
							default: wrap が trueの場合 0,false の場合は str を画面中央に表示する座標	
		wrap:boolean(opt)	長い文字列を自動的に折り返す default: false
							折り返す場合に、縦方向のフォントサイズの 1/5 に相当するピクセル分を空ける。
		cls:boolean(opt)	文字列描画前に画面をクリアする default: false
		font:string(opt)	フォント番号 0..9 の整数を指定 default: 以前に設定した値またはデバイス初期値の 2
		fg:number(opt)		文字色のカラーコードを指定する		default:15 (WHITE)
		bg:number(opt)		背景色のカラーコードを指定する		default:0  (BLACK)

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function build_frame_lcd_print(arr,str,x0,y0,wrap,cls,font,fg,bg)
	if not cls then cls = 0xff end
	if not font then font = 0xff end
	if not fg then fg = 0xff end
	if not bg then bg = 0xff end
	if not x0 then x0 = 0xff00 end
	if not y0 then y0 = 0xff00 end
	table.insert(arr,0x15)
	if wrap then 
		dxcmd_param_add_byte(arr,1)
	else
		dxcmd_param_add_byte(arr,0)
	end
	dxcmd_param_add_byte(arr,cls)
	dxcmd_param_add_byte(arr,font)
	dxcmd_param_add_byte(arr,fg)
	dxcmd_param_add_byte(arr,bg)
	dxcmd_param_add_word(arr,x0)
	dxcmd_param_add_word(arr,y0)
	local str_arr = str_to_tbl(str)
	table.insert(str_arr,0)
	for i,v in ipairs(str_arr) do
		table.insert(arr,v)
	end
end

function dxcmd_lcd_print(node,str,x0,y0,wrap,cls,font,fg,bg)
	if cls then dxcmd_lcd_clear(node) end
	local frame = {}
	build_frame_lcd_print(frame,str,x0,y0,wrap,0xff,font,fg,bg)
	return dxcmd_exec_stat_check(node,frame)
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) の LCD に日本語文字列を表示する。


関数定義
		dxcmd_lcd_jprint(node,str,x0,y0,wrap,cls,font,scr_w,scr_h,split,size,fg,bg)

パラメータ
		node:string			MGCP ノード名
		str:string			日本語文字列。ASCII 文字は同じ字形の全角文字に自動変換される。
							各文字のフォントデータが見つからなかった場合にはその文字は表示しない。
		x0:number(opt)		表示開始座標(X軸) default: 0
		y0:number(opt)		表示開始座標(Y軸) default: 0
		wrap:boolean(opt)	長い文字列を自動的に折り返す default: false
							折り返す時に、縦方向のフォントサイズの 1/5 に相当するピクセル分を空ける。
		cls:boolean(opt)	文字列描画前に画面をクリアする default: false
		font:string(opt)	グローバル共有エリアにロード済みの日本語フォントのチャンネル名 default: "SHINONOME_FONTX"
		scr_w:number(opt)	LCD 表示器の横サイズ default:M5Stack 320, M5Stick 160
		scr_h:number(opt)	LCD 表示器の縦サイズ default:M5Stack 240, M5Stick 80
		split:number(opt)	指定した文字数ごとに、複数の ダイレクトコマンドに分割して文字を表示する	default:3
		size:number(opt)	文字を size 倍に拡大して表示する	default:1
		fg:number(opt)		文字色のカラーコードを指定する		default:15 (WHITE)
		bg:number(opt)		背景色のカラーコードを指定する		default:0  (BLACK)

リターン値
		stat:boolean	true 成功、false 失敗

グローバル共有変数
		LCD に最後に表示した"文字"の次の表示開始予定座標をグローバル共有変数に保存する。
		前回表示した文字列に続けて表示したい場合に、次に表示開始座標を計算するときに利用できる。
		画面のサイズは考慮していないので折り返し表示が必要な場合は座標値の再計算が必要。

		キー名                    値
		-------------------------+---------------------------------------------------------
		MGCP_LCD_PREV_<node>	  表示した最後の文字の情報 (CSV形式で <x0>,<y0>,<fh>,<size>)
 
		<x0>	      表示した最後の文字の次の開始座標 X軸
		<y0>	      表示した最後の文字の次の開始座標 Y軸
		<fh>	      表示した最後の文字の縦方向のフォントサイズ
		<size>	      表示した最後の文字のフォント拡大倍率(size パラメータに指定したもの)

****************************************************************************************************
]]

function dxcmd_exec_lcd_cmd(node,bmp_cmd_arr,cls,size,fg,bg,last_x0,last_y0,font_w,font_h)
	if not fg then fg = 15 end
	if not bg then bg = 0 end
	if not size then size = 1 end
	local frame = {}
	table.insert(frame,0x16)
	if cls then
		dxcmd_param_add_byte(frame,bg)
	else
		dxcmd_param_add_byte(frame,0xff)
	end
	dxcmd_param_add_byte(frame,fg)
	dxcmd_param_add_byte(frame,bg)
	dxcmd_param_add_byte(frame,size)
	dxcmd_param_append(frame,bmp_cmd_arr)
	local stat = dxcmd_exec_stat_check(node,frame)
	if not stat then -- retry once
		wait_time(100)
		stat = dxcmd_exec_stat_check(node,frame)
	end
	if stat then
		set_shared_data("MGCP_LCD_PREV_" .. node,
						tostring(last_x0 + font_w) .. ',' .. 
						tostring(last_y0)  .. ',' ..
						tostring(font_h) .. ',' ..
						tostring(size))
	end
	return stat
end

function dxcmd_lcd_jprint(node,str,x0,y0,wrap,cls,font,scr_w,scr_h,split,size,fg,bg)
	fontx_try_init() -- FONTX ライブラリがまだ初期化されていない場合にはフォントをグローバル共有メモリにロードする
	if not x0 then x0 = 0 end
	if not y0 then y0 = 0 end
	if not wrap then wrap = false end
	if not cls then cls = false end
	if (not scr_w) or (not scr_h) then
		if not scr_w then scr_w = 160 end
		if not scr_h then scr_h = 80 end
		local dev_type = stat_check(get_shared_data("$TYPE_" .. node))
		if dev_type == "M5Stack" then
			scr_w = 320
			scr_h = 240
		end
		if dev_type == "M5Stick" then
			scr_w = 160
			scr_h = 80
		end
	end
	if not size then size = 1 end
	if not split then split = 3 end
	if not font then font = "SHINONOME_FONTX" end
	local stat, x = shmem_copy(font,14,2) 		-- FONTX2 形式のフォントサイズ部分を取得する
	if not stat then return false end
	local font_w = x[1]
	local font_h = x[2]
	local last_x0 = x0
	local last_y0 = y0
	local sp_h = math.floor((font_h * size) / 5)		-- 縦方向の文字間隔を計算
	local cstat,ch_arr = str_to_tbl2(str,2) 	-- UTF-8文字列を 1文字毎の配列に変換して且つ、文字コードをShift_JIS に変換
	if (not cstat) or (#ch_arr == 0) then return false end
	fontx_ascii_to_zenkaku(ch_arr)				-- ASCII コード部分のフォントが無い場合を想定して、同じ字形の全角文字に変換
	local bmp_cmd = {}
	local cmd_stat

	---------------------------------------------------------------------------------------------------------
	-- split で指定した文字数毎に lcd_bitmap コマンドを複数回コールする。
	---------------------------------------------------------------------------------------------------------
	local prev_split_idx = 0
	local split_idx						-- lcd_bitmap コマンドを複数回コールするときに、何回目のコールになるかを示す
	local first = true						-- 最初の lcd_bitmap コール時に画面クリア関連のフラグとして使用
	for k,v in ipairs(ch_arr) do
		split_idx = math.modf((k + split - 1)/split)
		if (prev_split_idx ~= split_idx) and (split_idx > 1) then
			cmd_stat = dxcmd_exec_lcd_cmd(node,bmp_cmd,cls and first,size,fg,bg,last_x0,last_y0,font_w,font_h)
			if not cmd_stat then return false end
			first = false
			bmp_cmd = {}
		end
		local gstat, fdata = shmem_get_fontx(font,v)	-- フォントデータ取得
		if not gstat then goto NEXT_CHAR_SPLIT end	-- フォントデータが無かった場合にはその文字をスキップ

		dxcmd_param_add_word(bmp_cmd,x0)
		dxcmd_param_add_word(bmp_cmd,y0)
		dxcmd_param_add_word(bmp_cmd,font_w)
		dxcmd_param_add_word(bmp_cmd,font_h)
		dxcmd_param_append(bmp_cmd,fdata)
		last_x0 = x0
		last_y0 = y0
		x0 = x0 + (font_w * size)
		if x0 >= scr_w then							-- このまま表示すると画面横方向にはみ出しそう？
			if not wrap then break end				-- wrap 指定が無いならそこで表示終了
			x0 = 0										-- wrap 指定ならば縦にずらして改行を演出
			y0 = y0 + sp_h + (font_h * size)
		end
		if (y0 + (font_h * size)) >= scr_h then break end		-- 表示すると画面縦方向にはみ出る見込みなら表示終了
::NEXT_CHAR_SPLIT::
		if k == #ch_arr then
			cmd_stat = dxcmd_exec_lcd_cmd(node,bmp_cmd,cls and first,size,fg,bg,last_x0,last_y0,font_w,font_h)
			if not cmd_stat then return false end
			bmp_cmd = {}
		end
		prev_split_idx = split_idx
	end
	if #bmp_cmd > 1 then -- 画面はみ出しのため途中で表示対象の文字追加を中止したときに、表示可能分の未転送文字列を描画
		cmd_stat = dxcmd_exec_lcd_cmd(node,bmp_cmd,cls and first,size,fg,bg,last_x0,last_y0,font_w,font_h)
	end
	return cmd_stat
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) の LCD に日本語文字列を表示する。
前回 LCD に表示した文字列の終端で改行して画面左端から表示する。
長い文字列は自動的に折り返して表示する。

関数定義
		dxcmd_lcd_jprintln(node,str,cls,font,scr_w,scr_h,split,size,fg,bg)

パラメータ
		node:string			MGCP ノード名
		str:string			日本語文字列。ASCII 文字は同じ字形の全角文字に自動変換される。
							各文字のフォントデータが見つからなかった場合にはその文字は表示しない。
		cls:boolean(opt)	文字列描画前に画面をクリアする default: false
		font:string(opt)	グローバル共有エリアにロード済みの日本語フォントのチャンネル名 default: "SHINONOME_FONTX"
		scr_w:number(opt)	LCD 表示器の横サイズ default:160
		scr_h:number(opt)	LCD 表示器の縦サイズ default:80
		split:number(opt)	指定した文字数ごとに、複数の ダイレクトコマンドに分割して文字を表示する	default:3
		size:number(opt)	文字を size 倍に拡大して表示する	default:1
		fg:number(opt)		文字色のカラーコードを指定する		default:15 (WHITE)
		bg:number(opt)		背景色のカラーコードを指定する		default:0  (BLACK)


リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function dxcmd_lcd_jprintln(node,str,cls,font,scr_w,scr_h,split,size,fg,bg)
	if not size then size = 1 end
	local stat, prev_info = get_shared_data("MGCP_LCD_PREV_" .. node)
	local prev_info_arr = csv_to_tbl(prev_info)
	if cls or (#prev_info_arr ~= 4) then 
		return dxcmd_lcd_jprint(node,str,0,0,true,cls,font,scr_w,scr_h,split,size,fg,bg)
	end
	local y0 = tonumber(prev_info_arr[2])
	local font_h = tonumber(prev_info_arr[3])
	local prev_size = tonumber(prev_info_arr[4])
	local sp_h = math.floor((font_h * prev_size) / 5)
	y0 = y0 + sp_h + (font_h * prev_size)
	return dxcmd_lcd_jprint(node,str,0,y0,true,cls,font,scr_w,scr_h,split,size,fg,bg)
end


--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD を指定した色で塗りつぶす

関数定義
		dxcmd_lcd_fill_screen(node,color)

パラメータ
		node:string			MGCP ノード名

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function build_frame_lcd_fill_screen(arr,color)
	table.insert(arr,0x10)
	dxcmd_param_add_byte(arr,color)
end

function dxcmd_lcd_fill_screen(node,color)
	local frame = {}
	build_frame_lcd_fill_screen(frame,color)
	return dxcmd_exec_stat_check(node,frame)
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD をクリアする(黒で塗りつぶす)

関数定義
		mgcp_lcd_clear(node)

パラメータ
		node:string			MGCP ノード名

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function dxcmd_lcd_clear(node)
	local frame = {}
	build_frame_lcd_fill_screen(frame,0)
	return dxcmd_exec_stat_check(node,frame)
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD に線分を描画する。

関数定義
		dxcmd_lcd_line(node,x0,y0,x1,y1[,color][,cls])

パラメータ
		node:string			MGCP ノード名
		x0:number			線分開始座標(X軸) 
		y0:number			線分開始座標(Y軸)
		x1:number			線分終了座標(X軸) 
		y1:number			線分終了座標(Y軸)
		color:number		描画色のカラーコードを指定する		default:15 (WHITE)
		cls:boolean			描画前に画面をクリアする default: false

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function build_frame_lcd_line(arr,x0,y0,x1,y1,color,cls)
	if not color then color = 15 end
	table.insert(arr,0x11)
	dxcmd_param_add_word(arr,x0)
	dxcmd_param_add_word(arr,y0)
	dxcmd_param_add_word(arr,x1)
	dxcmd_param_add_word(arr,y1)
	dxcmd_param_add_byte(arr,color)
end

function dxcmd_lcd_line(node,x0,y0,x1,y1,color,cls)
	if cls then dxcmd_lcd_clear(node) end
	local frame = {}
	build_frame_lcd_line(frame,x0,y0,x1,y1,color,cls)
	return dxcmd_exec_stat_check(node,frame)
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD に円を描画する。

関数定義
		dxcmd_lcd_circle(node,x,y,r,[,color][,cls][,fill])

パラメータ
		node:string			MGCP ノード名
		x:number			座標(X軸) 
		y:number			座標(Y軸)
		r:number			半径 
		color:number		描画色のカラーコードを指定する		default:15 (WHITE)
		cls:boolean			描画前に画面をクリアする default: false
		fill:boolean		図形内部を塗りつぶす default: false

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function build_frame_lcd_circle(arr,x,y,r,color,cls,fill)
	if not color then color = 15 end
	table.insert(arr,0x12)
	if fill then
		dxcmd_param_add_byte(arr,1)
	else
		dxcmd_param_add_byte(arr,0)
	end
	dxcmd_param_add_word(arr,x)
	dxcmd_param_add_word(arr,y)
	dxcmd_param_add_word(arr,r)
	dxcmd_param_add_byte(arr,color)
end

function dxcmd_lcd_circle(node,x,y,r,color,cls,fill)
	if cls then dxcmd_lcd_clear(node) end
	local frame = {}
	build_frame_lcd_circle(frame,x,y,r,color,cls,fill)
	return dxcmd_exec_stat_check(node,frame)
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD に四角を描画する。

関数定義
		dxcmd_lcd_rect(node,x,y,w,h [,color][,cls][,fill])

パラメータ
		node:string			MGCP ノード名
		x:number			座標(X軸) 
		y:number			座標(Y軸)
		w:number			四角形の幅
		h:number			四角形の高さ
		color:number		描画色のカラーコードを指定する		default:15 (WHITE)
		cls:boolean			描画前に画面をクリアする default: false
		fill:boolean		図形内部を塗りつぶす default: false

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function build_frame_lcd_rect(arr,x,y,w,h,color,cls,fill)
	if not color then color = 15 end
	table.insert(arr,0x14)
	if fill then
		dxcmd_param_add_byte(arr,1)
	else
		dxcmd_param_add_byte(arr,0)
	end
	dxcmd_param_add_word(arr,x)
	dxcmd_param_add_word(arr,y)
	dxcmd_param_add_word(arr,w)
	dxcmd_param_add_word(arr,h)
	dxcmd_param_add_byte(arr,color)
end

function dxcmd_lcd_rect(node,x,y,w,h,color,cls,fill)
	if cls then dxcmd_lcd_clear(node) end
	local frame = {}
	build_frame_lcd_rect(frame,x,y,w,h,color,cls,fill)
	return dxcmd_exec_stat_check(node,frame)
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD に三角を描画する。

関数定義
		dxcmd_lcd_triangle(node,x0,y0,x1,y1,x2,y2[,color][,cls])

パラメータ
		node:string			MGCP ノード名
		x0:number			頂点1座標(X軸) 
		y0:number			頂点1座標(Y軸)
		x1:number			頂点2座標(X軸) 
		y1:number			頂点2座標(Y軸)
		x2:number			頂点3座標(X軸) 
		y2:number			頂点3座標(Y軸)
		color:number		描画色のカラーコードを指定する		default:15 (WHITE)
		cls:boolean			描画前に画面をクリアする default: false

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function build_frame_lcd_triangle(arr,x0,y0,x1,y1,x2,y2,color,cls,fill)
	if not color then color = 15 end
	table.insert(arr,0x13)
	if fill then
		dxcmd_param_add_byte(arr,1)
	else
		dxcmd_param_add_byte(arr,0)
	end
	dxcmd_param_add_word(arr,x0)
	dxcmd_param_add_word(arr,y0)
	dxcmd_param_add_word(arr,x1)
	dxcmd_param_add_word(arr,y1)
	dxcmd_param_add_word(arr,x2)
	dxcmd_param_add_word(arr,y2)
	dxcmd_param_add_byte(arr,color)
end

function dxcmd_lcd_triangle(node,x0,y0,x1,y1,x2,y2,color,cls,fill)
	if cls then dxcmd_lcd_clear(node) end
	local frame = {}
	build_frame_lcd_triangle(frame,x0,y0,x1,y1,x2,y2,color,cls,fill)
	return dxcmd_exec_stat_check(node,frame)
end

--[[
****************************************************************************************************

MGCP デバイス(M5StickC/M5Stack) LCD に点を描画する。

関数定義
		dxcmd_lcd_pixel(node,x,y,[,color][,cls])

パラメータ
		node:string			MGCP ノード名
		x:number			座標(X軸) 
		y:number			座標(Y軸)
		color:number		描画色のカラーコードを指定する		default:15 (WHITE)
		cls:boolean			描画前に画面をクリアする default: false

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function build_frame_lcd_pixel(arr,x0,y0,color,cls)
	if not color then color = 15 end
	table.insert(arr,0x17)
	dxcmd_param_add_word(arr,x0)
	dxcmd_param_add_word(arr,y0)
	dxcmd_param_add_byte(arr,color)
end

function dxcmd_lcd_pixel(node,x0,y0,color,cls)
	if cls then dxcmd_lcd_clear(node) end
	local frame = {}
	build_frame_lcd_pixel(frame,x0,y0,color,cls)
	return dxcmd_exec_stat_check(node,frame)
end


